package org.jharks.workflow;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.ListIterator;


/**
 * Represents the context within which a question was answered.  The context 
 * simply wraps the answer properties. The context is created when responses
 * are added to the <CODE>SessionHistory</CODE> instance as <CODE>Answer</CODE> 
 * instances within the <CODE>BaseWorkFlowManagerImpl</CODE>. The properties are placed
 * on the context by the <CODE>ContextInspector<CODE> utility class which 
 * inspects the stack of queues in a top down order.  The first item in the 
 * property set will therefore be the top item on the stack.  If you are in 
 * a "Medication" loop nested withi a "Problem Class" loop, then (for example)
 * <pre>
 *   property 1 in the context would be:
 *         "Medication"="Motrin" 
 *   and property 2 would be 
 *         "Problem Class"="Analgesic".
 * </pre> 
 *  
 * @author Michael J. Rizzo
 * @version 1.0
 */
public final class AnswerContext implements Cloneable, Serializable {

	private static final long serialVersionUID = 1L;

	/** The collection of properties available within the answer context. */
    private ArrayList<AnswerProperty> answerProperties; 
    
    /**
     * Default constructor
     */
    public AnswerContext() {
        answerProperties = new ArrayList<AnswerProperty>();
    }
    
    /**
     * Add an <CODE>AnswerProperty</CODE> to the context.
     * @param prop an <CODE>AnswerProperty</CODE> instance to be added to the 
     *             context.
     */
    public void addProperty(AnswerProperty prop) {
        answerProperties.add(prop);
    }
    
    /**
     * Adds the passed <CODE>AnswerProperty<CODE> instances to the context.
     * @param props a collection of <CODE>AnswerProperty</CODE> objects.
     */
    public void addProperties(Collection<AnswerProperty> props) {
        answerProperties.addAll(props);
    }

    /**
     * Returns the <CODE>AnswerProperty</CODE> located within the context
     * at the specified index.
     * @param index the index of the element to return.
     * @return the <CODE>AnswerProperty</CODE> instance at the specified 
     *         position in the list. 
     */
    public AnswerProperty getProperty(int index) {
        return answerProperties.get(index);
    }
    
    /**
     * Returns an iterator for the <CODE>AnswerProperty<CODE> objects in
     * the context.
     * @return an iterator of the properties in the context (in proper sequence).
     */
    public ListIterator<AnswerProperty> getPropertyIterator() {
         return answerProperties.listIterator();
    }
    
    /**
     * Returns the number of properties that have been set.
     * @return the number of properties set in the context.
     */
    public int getPropertySize() {
        return answerProperties.size();
    }
    
    /**
     * Returns a string representation of the context object. The format of
     * the string follows the format:
     * <br><br>
     *   &lt;<i>first property-name</i>:<i>property-value</i>&gt;
     *   &lt;<i>second property-name</i>:<i>property-value</i>&gt;
     * @return a string representation of the context objects.
     */
    public String toString() {
        if (answerProperties == null || answerProperties.size() == 0)
            return "";
        
        StringBuffer sb = new StringBuffer();
        for (Iterator<AnswerProperty> it = answerProperties.iterator (); it.hasNext (); ) {
          AnswerProperty prop = it.next ();
          sb.append("<")
            .append(prop.getName())
            .append(":")
            .append(prop.getStringValue())
            .append(">");
        }

        return sb.toString();
    }
    
    /**
     * Determines if the objects are equal by comparing property set.
     *
     * @return true if property sets are equal (or both empty)
     */
    public boolean equals(Object o)
    {
        
        if (o != null && (o.getClass().equals(this.getClass()))) {
        if (this == o) return true;
        AnswerContext context = (AnswerContext) o;
        if (answerProperties.isEmpty() && context.answerProperties.isEmpty())
            return true;
        if (answerProperties.isEmpty() || context.answerProperties.isEmpty())
            return false;
        if (! (answerProperties.size() == context.answerProperties.size())) 
            return false; 
        
        for(int i=0; i < answerProperties.size(); i++) {
            Object prop1 = answerProperties.get(i);
            Object prop2 = context.answerProperties.get(i);
            if (!(prop1 instanceof AbstractProperty) ||
               !(prop2 instanceof AbstractProperty ))
                return false;
            AbstractProperty aprop1 = (AbstractProperty) prop1;
            AbstractProperty aprop2 = (AbstractProperty) prop2;
            if(!(aprop1.equals(aprop2))) return false;
        }
            
        return true;
        
        } else {
            return false;
        }
    }
    
    /**
     * Returns hash code as summation of property hash codes.
     * @return hash code of AnswerContext.
     */
    public int hashCode() {
        int result = 0;
        if (!answerProperties.isEmpty()) {  
            for(int i=0; i < answerProperties.size(); i++) {
                result = result + 
                  (((AbstractProperty) answerProperties.get(i)).hashCode());
            }
        }
        return result;
    }
    
    /**
     * @see Object#clone()
     */
    public Object clone() 
    {
        AnswerContext newContext = null;
        try {
          newContext = (AnswerContext) super.clone();
          newContext.answerProperties = this.answerProperties;
        } catch (CloneNotSupportedException cne) {
          //TODO: handle exception more gracefully.  
            System.out.println("error cloning" + cne.getMessage());
            cne.printStackTrace();
        }
        return newContext;
    }

}
