package org.jharks.workflow;

import java.util.MissingResourceException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jharks.workflow.exception.MissingValueHandlerException;
import org.jharks.workflow.util.WorkFlowUtilities;


/**
 * Factory class that creates value handlers.
 * @see org.jharks.workflow.AnswerValueHandler
 * 
 * @author Michael J. Rizzo
 * @version 1.0
 */
public class AnswerValueHandlerFactory {
   
    /** The logger for the module. */
    protected static Log logger = LogFactory.getLog(AnswerValueHandlerFactory.class); 
    
    /** Suffix to be added to class number for key lookup. */
    public final static String HANDLER_SUFFIX = "-handler";
    
    /**
     * Default Constructor
     */
    public AnswerValueHandlerFactory() {
    }
    
    /**
     * Factory mehtod that determines appropriate value handler for
     * value object.
     * @param valueObject the value object to be inspected.
     * @return handle to <code>AnswerValueHandler</code> implementation.
     * @throws ClassNotFoundException 
     * @throws IllegalAccessException
     * @throws InstantiationException
     * @throws MissingValueHandlerException
     */
    public static AnswerValueHandler getValueHandler(Object valueObject) 
    throws ClassNotFoundException, 
           IllegalAccessException, 
           InstantiationException,
           MissingValueHandlerException {

      WorkFlowConfig wc = WorkFlowConfig.getInstance(
              WorkFlowConstants.PROP_FILE_NAME);  
      
      String fullClassName = WorkFlowUtilities.getFullClassName(
              valueObject.getClass()); 
      
      if (logger.isDebugEnabled())
          logger.debug("AnswerValueHandler: class name = " + fullClassName);
      
      String handlerClass = null;
      
      try {
          handlerClass = wc.getProperty(fullClassName);
      } catch (MissingResourceException mre) {
        //No key found for full class try generic
      }
      
      if (handlerClass == null) {
        try {
            handlerClass = getGenericType(valueObject.getClass(), wc);
        } catch (MissingResourceException mre) {
          throw new MissingValueHandlerException("No handler found for: " +
                  fullClassName);
        }
      }
      
      if (logger.isDebugEnabled())
          logger.debug("AnswerValueHandler: handler class = " + handlerClass);
      
      return (AnswerValueHandler) Class.forName(handlerClass).newInstance();
    }
    
    /**
     * Method that returns reference to a handler class by looking up
     * a key within the workflow configuration. The key is based on 
     * the classname and <code>HANDLER_SUFFIX</code>.
     * @param c the passed <code>Class</code.
     * @param pl the workflow configuration instance.
     * @return the handler class name for the specified class.
     * @throws MissingResourceException if lookup key can not be found.
     */
    private static String getGenericType(Class<?> c, WorkFlowConfig wc)
        throws MissingResourceException {
        String className = WorkFlowUtilities.getFullClassName(c);
        
        String genericHandleClass = wc.getProperty(className + HANDLER_SUFFIX);
        
        return genericHandleClass;
    }
}
