package org.jharks.workflow;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.Collection;
import java.util.EmptyStackException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.apache.commons.digester.Digester;
import org.apache.commons.digester.xmlrules.DigesterLoader;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jharks.workflow.exception.ConfigurationException;
import org.jharks.workflow.exception.StateInitializationException;
import org.jharks.workflow.exception.WorkFlowException;
import org.jharks.workflow.exception.WorkFlowActionException;
import org.jharks.workflow.handlers.SimpleErrorHandler;
import org.jharks.workflow.util.ObjectCopier;
import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;


/**
 * The org.jharks.workflow.BaseWorkFlowManagerImpl class is responsible 
 * for controlling the navigation path through the application logic.  The 
 * BaseWorkFlowManagerImpl functions as the state context and keeps a reference to 
 * the current state of the application instance.  The BaseWorkFlowManagerImpl will 
 * also be responsible for building the state objects, transition objects and 
 * transition rule objects that will be available within the application 
 * workflow.   
 * <br><br>
 * The available states, transitions and transition rules will be configurable
 * through a XML based file.  The BaseWorkFlowManagerImpl will utilize parsing 
 * technology supplied within the Apache Digester library to parse the XML file
 * and instantiate the necessary workflow objects.
 * 
 * @author Michael J. Rizzo
 * @version 1.0
 */

public class BaseWorkFlowManagerImpl implements WorkFlowManager{
    
    /** Constant representing start state label. */
    private final static String START = "start";
    
    /** The logger for the module. */
    protected Log logger = LogFactory.getLog(getClass().getName());
    
    /** String representing startState for BaseWorkFlowManagerImpl. */
    private String startState;
    
    /** Map of available states. */
    private HashMap<String, State> states = new HashMap<String, State>();
    
    /** Map of loaders used by workflow manager to queue information by different criteria. */
    private HashMap<String, Loader> loaders = new HashMap<String, Loader>();
    
    /** Custom entity resolver. */
    private EntityResolver resolver;
    
    /** Default public constructor. */
    public BaseWorkFlowManagerImpl() {      
    }

    /**
     * {@inheritDoc}
     */
    public void setCustomEntityResolver(EntityResolver resolver) {
        this.resolver = resolver;
    }
    
    /**
     * {@inheritDoc}
     */
    public EntityResolver getCustomEntityResolver() {
        return this.resolver;
    }
    
    /**
     * The init method is used to initialize the BaseWorkFlowManagerImpl. This method 
     * creates the singleton instance, performs other initial configurations 
     * and digests the XML configuration file associated with the BaseWorkFlowManagerImpl, 
     * creating all the necessary workflow objects.
     */
    public void init()
        throws ConfigurationException {
        this.loadStates(null);
        this.setStartState("start", false, null);
    }
    
    /**
     * The init method is used to initialize the BaseWorkFlowManagerImpl. This method 
     * performs other initial configurations and digests the XML configuration file associated 
     * with this instance of BaseWorkFlowManagerImpl, creating all the necessary workflow objects.
     * @param workflowDefinitionFileName the name of the workflow definition file to be digested.
     */
    public void init(String workflowDefinitionFileName)
        throws ConfigurationException {
        this.loadStates(workflowDefinitionFileName);
        this.setStartState("start", false, null);
    }
    
    /**
     * {@inheritDoc}
     */
    public void init(String workflowDefinitionName, InputStream workflowDefinitionStream) 
        throws ConfigurationException {
    	
    	this.loadStates(workflowDefinitionName, workflowDefinitionStream);
    	this.setStartState("start", false, null);
    }
    
    /**
     * The getNextMapping method is called by the <CODE>ViewController</CODE> 
     * in order to receive the next view in the application.  The 
     * <CODE>ViewMapping</CODE> represents a mapping object to be used by the
     * <CODE>ViewController</CODE> to determine the next layout or control to 
     * display within the UI.  The getNextView method implementation calls the 
     * internal process method. 
     * 
     * @param input the <CODE>ResponseSet</CODE> containing submitted input to be 
     *        processed.
     * @return The determined <CODE>ViewMapping</CODE> to the caller.
     */
    public ViewMapping getNextMapping(ResponseSet input, 
            WorkFlowContext flowContext) throws WorkFlowException {
        State activeState = flowContext.getActiveState();
        if (activeState != null) {
          if (logger.isDebugEnabled()) 
            logger.debug("getNextMapping(): Active state: " +
                 "stateKey =[ " + activeState.getName() + " ]" );
        }
        
        String stateKey = process(input, flowContext);
        
        //allow retrieval of default state if no transition applied
        if (stateKey == null && activeState.getDefaultNextState() != null ) { 
           stateKey = activeState.getDefaultNextState();
           if (logger.isDebugEnabled()) 
               logger.debug("No transition applied using default next state " +
                    "if exists: stateKey =[ " + stateKey + " ]" );
            
        }
        
        if (stateKey != null) {
            if (logger.isDebugEnabled()) 
                logger.debug("stateKey =[ " + stateKey + " ]" );  
          State detState = (State) states.get(stateKey);
          
          //new lynch pin for auto forward.
          detState = initState(detState, flowContext);
          if (logger.isDebugEnabled()) 
              logger.debug("state after init = [ " + detState.getName() + " ]" );
          
          flowContext.setActiveState(detState);
        
          postState(activeState, flowContext);
          
          ViewMapping viewMapping = null;
          if (detState.isEndState()) {
        	  viewMapping = new ViewMapping("jharksWorkflowEndView", false);
          } else if (detState.getViewName() != null) {
              viewMapping = new ViewMapping(detState.getViewName(), detState.isAllowBackNavigation());
          }
          viewMapping.setViewProperties(flowContext.getHistory().getProperties());
      
          return viewMapping;
        }
        
        return null;
    }
    
    /**
     * Starts the configured workflow at the defined start
     * state.
     * @throws a <code>WorkFlowException</code> if <code>WorkFlowManager</code>
     *         fails to start the specified state and determine the next view.
     * @return The start state configured for this workflow.
     */
    public ViewMapping start(WorkFlowContext flowContext) 
      throws WorkFlowException {
        State start = (State) states.get(this.startState); 
        initState(start, flowContext);
        
        flowContext.setActiveState(start);
        return new ViewMapping(start.getViewName()); 
    }

    /**
     * Retrieve all configured states.
     * @return the available configured states. 
     */
    public HashMap<String, State> getStates() {
        return this.states;
    }

    /**
     * Set states using preconfigured map.
     * @param value a map of states.
     */
    public void setStates(HashMap<String, State> value) {
        states = value;
    }

    /**
     * Return the key set of available states.
     * @return the key set for the map of states.
     */
    public Set<String> statesKeySet() {
        return states.keySet();
    }

    /**
     * Return the collection of values within the state map.
     * @return the values objects contained within the state map.
     */
    public Collection<State> statesValues() {
        return states.values();
    }

    /**
     * Check if the state map contains the key.
     * @param key the key object to see if contained within the key set.
     * @return true if this map contains a mapping for the specified key.
     */
    public boolean statesContainsKey(Object key) {
        return states.containsKey(key);
    }

    /**
     * Check if the value is mapped to a key within the map.
     * @param value whose prsence in this map is to be tested.  
     * @return true if this map maps one or more keys to specified value.
     */
    public boolean statesContainsValue(Object value) {
        return states.containsValue(value);
    }

    /**
     * Returns the value to which the specified key is mapped.
     * @param key key whose associated value is to be returned.
     * @return the value to which this key is mapped, if key is not present
     *         returns null.
     */
    public State getState(Object key) {
        return states.get(key);
    }

    /**
     * Removes this mapping for the key if present within the map.
     * @param key key whose mapping is to be removed from the map.
     */
    public State removeState(Object key) {
        return states.remove(key);
    }
    
    /**
     * Set a start state through the digester.
     * @param value the string representation of the start state
     *        name;
     */
    public void setStartState(String value) {
        this.startState = value;
    }
    
    /**
     * Set or reset the start state through programming code.
     * @param value the string representation of the start state
     *        name;
     * @param initialize the parameter that specifies whether initialization
     *        should be performed or not.
     * @throws exception if error occurs initializing the specified start 
     *         state.
     */
    public void setStartState(String value, boolean initialize, 
            WorkFlowContext flowContext) throws ConfigurationException {
        this.startState = value;
        State st = null;
        try {
          if (initialize) {
            st = initState(getState(value), flowContext);
          } else {
            st = getState(value);    
          }
          
          if (flowContext != null) { 
                flowContext.setActiveState(st);
          } 
        }  catch (StateInitializationException sie) {
                throw new ConfigurationException(sie);
        }
        
    }
    
    /**
     * Method used by digester to add a state object to workflow.
     * @param st the <CODE>StateImpl</CODE> instance created by the digester.
     */
    public void addState(StateImpl st) {
        if (st.getName().equalsIgnoreCase(START)) 
            this.startState = st.getName();
        putState(st);
    }
    
    /**
     * Method that adds a <CODE>StateImpl</CODE> instance to the 
     * <CODE>Map</CODE> of states.
     * @param st the <CODE>StateImpl</CODE> instance to be added.
     */
    protected void putState(StateImpl st) {
        states.put(st.getName(), st);
    }
    
    /**
     * Clear all states previously populated.  
     */
    public void clearStates() {
        states.clear();
    }
    
    /**
     * Method used by digester to add a <code>Loader</code> object to workflow loader map.
     * The key for the loader is the alias that was configured for the loader in the 
     * workflow.xml file.
     * @param l the <CODE>Loader</CODE> instance created by the digester.
     */
    public void addLoader(Loader l) {
        if(logger.isDebugEnabled())
            logger.debug("Adding loader to workflow manager: Alias = [" 
                    + l.getLoaderAlias() + "]" );
        loaders.put(l.getLoaderAlias(), l);
    }
    
    /**
     * Private method used by workflow to lookup loader instance. 
     * @param loaderAlias the alias key to lookup the loader.
     * @return the mapped <code>Loader</code> instance.
     */
    private Loader lookupLoader(String loaderAlias) {
       return loaders.get(loaderAlias); 
    }
    
    /**
     * The process method is the real workhorse method of the class.   
     * The process method handles the logic to actually determine the 
     * next mapping view in the workflow that will be returned to the 
     * ViewController.  The method will iteratively process the 
     * Transitions associated with the active state applying the 
     * associated RuleSet and or Rules till the appropriate Transition 
     * is identified.
     * 
     * @param input the Object containing submitted input to be 
     *        processed.
     * @throws all lower level processing exceptions as a 
     *         <code>WorkFlowException</code>
     * @return String the string repreenting the ViewMapping to be
     *        created and returned to caler.
     */
    private String process(ResponseSet input, WorkFlowContext flowContext) 
      throws WorkFlowException {
        
        String stateKey = null;
        boolean processResult = false;
        
        SessionHistory workflowSessionHistory = flowContext.getHistory();
        if(input != null) { 
            if (logger.isDebugEnabled()) {
                logger.debug(getClass().getName() + ": Adding responses:");
                Iterator<String> i = input.getQuestionIds().iterator();
                while (i.hasNext()) {
                    logger.debug("Question ID = [" + i.next() + "]");
                }
                        
            }
            workflowSessionHistory.addResponses((ResponseSet) input);
        }
        
        //If Snapshot mode is on then a snapshot of the SessionHistory with 
        //the last responses will be added into a map in memory.
        if (flowContext.isWorkFlowHistorySnapshots())
      	  persistSnapshotToMemory(workflowSessionHistory, flowContext);
        
        List<Transition> transitions = flowContext.getActiveState().getTransitions();
        
        for (int i = 0; i < transitions.size(); i++ ) {
         Transition tr = (Transition) transitions.get(i);
         
         processResult = process(tr, flowContext);
            
         if (processResult) {
            if (tr instanceof StateTransition)
              postProcess((StateTransition) tr, input, flowContext);
            stateKey = tr.getNextState();
            break;   
         }
        }
            
        return stateKey;
    }
    
    /**
     * Method to iterate over collection set and decide transition.
     * @param tr the <CODE>Transition</CODE> instance.
     */
    private boolean process(Transition tr, WorkFlowContext flowContext) {
         
         if(tr.getRules().size() > 0) {
             
             List<Rule> rules = tr.getRules();
             
             for (int i = 0; i < rules.size(); i++ ) {
              Rule r = (Rule) rules.get(i); 
           
               if( r.checkRule(flowContext.getHistory()) ) {
                 return true; 
               }
             }
         } else {
             //No rules set on tranisition so must apply
             return true;
         }

      return false; 
    }
    
    /**
     * Method executes post processes following an identified 
     * transition point.    
     * @param tr the <CODE>Transition</CODE> instance.
     */
    private void postProcess(Transition tr, ResponseSet input, 
            WorkFlowContext flowContext) throws WorkFlowException {
        //TODO: look at basing actions on interfaces.
        if (tr instanceof Queueable) {
           Queueable qtr = (Queueable) tr;
           
           if (qtr.isQueueInput()) {
            
             if (qtr.getQueueBy() != null) {
              if (logger.isDebugEnabled())
                logger.debug("postProcess: queuable transition: queue alias = ["
                      + qtr.getQueueBy() + "]");
          
              Loader l = lookupLoader(qtr.getQueueBy());
              l.load(input, flowContext.getHistory(), qtr);
             } else {
              throw new WorkFlowException("Error occured processing " +
                    "transition: an alias must be specified for the queue.");
             }
           }
        }
        
        if (tr instanceof Cleanable) {
          Cleanable ctr = (Cleanable) tr;
          if (ctr.isCleanUp()) {
        	SessionHistory history = flowContext.getHistory();
            history.removeProperty(ctr.getInputProperty());
            
            try {
              QueueWrapper item = (QueueWrapper) history.peek();
              if (item.queueSize() == 0 && 
                    item.getElementType().equals(ctr.getInputProperty()))
                  history.popStack();
            } catch (EmptyStackException ese) {
                if (logger.isDebugEnabled())
                    logger.debug("postProcess: clean up: stack is empty," 
                            + "EmptyStackException caught");
            }
          }
        }
        
        postTransition(tr, flowContext);
    }
     
    /**
     * Performs actual work of digesting the XML configuration file associated 
     * with the BaseWorkFlowManagerImpl and creating all the necessary workflow objects.
     * @param workflowDefinitionFileName the name of the workflow definition file to be digested.  
     */
    private void loadStates(String workflowDefinitionFileName) throws ConfigurationException {
        
        if (logger.isDebugEnabled()) 
            logger.debug("WorkFlowManger: loading states...");
        
        String configFile = null;
        
        WorkFlowConfig wc = WorkFlowConfig.getInstance();
        
        if (workflowDefinitionFileName == null || workflowDefinitionFileName.trim().equals("")) {
          configFile = wc.getProperty("workflow-file");
        } else {
          configFile = workflowDefinitionFileName;
        }
        
        if (logger.isDebugEnabled()) 
            logger.debug("WorkFlowManger: configFile set to: [ " 
                    + configFile + " ]");
        
        URL rulesFileName = BaseWorkFlowManagerImpl.class.getResource("workflowRules.xml");
         
        Digester digester = null;
        try {
        digester = DigesterLoader.createDigester(rulesFileName);    
        digester.setValidating( true );
        digester.setNamespaceAware(true);
        
        digester.setProperty("http://java.sun.com/xml/jaxp/properties/schemaLanguage", "http://www.w3.org/2001/XMLSchema");
        digester.setErrorHandler( new SimpleErrorHandler() );
        
        digester.push( this );
        
        URL fileUrl = this.getClass().getResource(configFile);
        if (logger.isDebugEnabled())
            logger.debug("File URL " + fileUrl.toExternalForm());
        InputSource is = new InputSource(fileUrl.toExternalForm());
        InputStream in = this.getClass().getResourceAsStream(configFile);
        is.setByteStream(in);
        
        //TODO: re-examine: convert dtd to schema and then how does this get impacted.
        //EntityResolver resolver = getCustomEntityResolver();
        //if (resolver != null) {
        //    digester.setEntityResolver(resolver);
        //} else {
        //    digester.setEntityResolver(new WorkFlowEntityResolver());
        //}
        
        digester.parse(is);
        } catch (IOException ioe) {
            if (logger.isErrorEnabled()) {
                logger.error("An error occurred while initializing the " +
                        "WorkFlow Manager", ioe);    
            }
            throw new ConfigurationException("Error occurred while " +
                    "initializing the WorkFlow Manager " + configFile, ioe); 
        } catch (SAXException saxe) {
            if (logger.isErrorEnabled()) {
                logger.error("An error occurred while parsing the " +
                        configFile, saxe);
            }
            throw new ConfigurationException("Error occurred while parsing " +
                    "the workflow configuration file: " + configFile, saxe); 
        }
        digester.clear();
    }
    
    /**
     * Performs actual work of digesting the XML configuration associated 
     * with the BaseWorkFlowManagerImpl and creating all the necessary workflow objects.
     * @param workflowDefinitionName the name associated withthe workflow definition stream to be digested.
     * @param workflowDefinitionStream the workflow stream to be processed.  
     */
    private void loadStates(String workflowDefinitionName, InputStream workflowDefinitionStream) throws ConfigurationException {
    
    	URL rulesFileName = BaseWorkFlowManagerImpl.class.getResource("workflowRules.xml");
        
    	Digester digester = null;
        try {
          digester = DigesterLoader.createDigester(rulesFileName);    
          digester.setValidating( true );
          digester.setNamespaceAware(true);
         
          digester.setProperty("http://java.sun.com/xml/jaxp/properties/schemaLanguage", "http://www.w3.org/2001/XMLSchema");
          digester.setErrorHandler( new SimpleErrorHandler() );
         
          digester.push( this );
    	  digester.parse(workflowDefinitionStream);
        } catch (IOException ioe) {
          if (logger.isErrorEnabled()) {
            logger.error("An error occurred while initializing the " +
                    "WorkFlow Manager", ioe);    
          }
          throw new ConfigurationException("Error occurred while " +
                "initializing the WorkFlow Manager. Problem processing the stream " + workflowDefinitionName, ioe); 
        } catch (SAXException saxe) {
          if (logger.isErrorEnabled()) {
            logger.error("An error occurred while parsing the passed workflow stream " +
                    workflowDefinitionName, saxe);
          }
          throw new ConfigurationException("Error occurred while parsing " +
                "the workflow configuration stream: " + workflowDefinitionName, saxe); 
        }
      digester.clear();
    }
    
    /**
     * Performs initialization activities on a configured state.
     * @param st The state to initialize.
     * @return state following the transition.
     * @throws a state initialization exception if an error occurs performing
     *         an init block on a <code>State</code>.
     */
    private State initState(State st, WorkFlowContext flowContext) 
      throws StateInitializationException {
      
      try { 
          
      
      if (st.getInitializer() != null && (! st.isInitialized()) ) { 
        
         if (logger.isDebugEnabled())
             logger.debug("Initializing state " + st.getName());
         
         if(st.getInitializer().getActions().size() > 0) {
             List<Action> actions = st.getInitializer().getActions();
             
             if (logger.isDebugEnabled())
                 logger.debug("Actions to be processed " + actions.size());
             
             for (int i = 0; i < actions.size(); i++ ) {
              Action action = (Action) actions.get(i); 
           
              action.apply(flowContext);
             }
         } 
         
         if(st.getInitializer().getRules().size() > 0) {
             List<Rule> rules = st.getInitializer().getRules();
             
             if (logger.isDebugEnabled())
                 logger.debug("Initializing state: Rules to be processed " 
                         + rules.size());
             
             for (int i = 0; i < rules.size(); i++ ) {
              Rule r = (Rule) rules.get(i); 
           
              r.checkRule(flowContext.getHistory());  
             }
         }
         
         
         if (st.getInitializer().getTransitions().size() > 0) {
             List<Transition> transitions =  st.getInitializer().getTransitions();
             
             boolean processResult = false;
             
             if (logger.isDebugEnabled())
                 logger.debug("Initializing state: Transitions to be processed " 
                         + transitions.size());
             
             for (int i = 0; i < transitions.size(); i++ ) {
                 Transition tr = (Transition) transitions.get(i);
                 
                 processResult = process(tr, flowContext);
                    
                 if (processResult) {
                    String stateKey = tr.getNextState();
                    State detState = (State) states.get(stateKey);
                    postTransition(tr, flowContext);
                    //recursion will recurse as long as applied transion
                    //and new state has a init block
                    return initState(detState, flowContext);   
                 }
             }
         }
      }
      
      } catch (WorkFlowException wfe) {
          throw new StateInitializationException(wfe.getMessage(), wfe);
      }
      return st;
    }
    
    /**
     * Executes a post process block on a transition.
     * @param tr the <code>Transition</code> to check for a post processor.
     * @throws an exception if error occurs processing post process block on a
     *         transition.
     */
    private void postTransition(Transition tr, WorkFlowContext flowContext)  
      throws WorkFlowException {
        if (tr.getPostProcessor() != null ) { 
            
             if (logger.isDebugEnabled())
                 logger.debug("Post transition processor executing: " +
                        "post transition " + tr.getName());
             
             if(tr.getPostProcessor().getActions().size() > 0) {
                 List<Action> actions = tr.getPostProcessor().getActions();
                 
                 if (logger.isDebugEnabled())
                     logger.debug("Actions to be processed " + actions.size());
                 
                 for (int i = 0; i < actions.size(); i++ ) {
                  Action action = (Action) actions.get(i); 
               
                  action.apply(flowContext);
                 }
             } 
             
             if(tr.getPostProcessor().getRules().size() > 0) {
                 List<Rule> rules = tr.getPostProcessor().getRules();
                 
                 if (logger.isDebugEnabled())
                     logger.debug("Post transition processor executing: " +
                            "Rules to be processed " 
                             + rules.size());
                 
                 for (int i = 0; i < rules.size(); i++ ) {
                  Rule r = (Rule) rules.get(i); 
               
                  r.checkRule(flowContext.getHistory());  
                 }
             }
        }
    }
    
    /**
     * Executes a post processor block on a state.
     * @param st the <code>StateImpl</code> to check for a post processor.
     * @throws an exception if error occurs processing post process block on a
     *         transition.
     */
    private void postState(State st, WorkFlowContext flowContext) 
      throws WorkFlowException {
        if (st.getPostProcessor() != null ) { 
            
             if (logger.isDebugEnabled())
                 logger.debug("Post state processor executing: post state " 
                         + st.getName());
             
             if(st.getPostProcessor().getActions().size() > 0) {
                 List<Action> actions = st.getPostProcessor().getActions();
                 
                 if (logger.isDebugEnabled())
                     logger.debug("Actions to be processed " + actions.size());
                 
                 for (int i = 0; i < actions.size(); i++ ) {
                  Action action = (Action) actions.get(i); 
               
                  action.apply(flowContext);
                 }
             } 
             
             if(st.getPostProcessor().getRules().size() > 0) {
                 List<Rule> rules = st.getPostProcessor().getRules();
                 
                 if (logger.isDebugEnabled())
                     logger.debug("Post state processeor executing: " +
                            "Rules to be processed " 
                             + rules.size());
                 
                 for (int i = 0; i < rules.size(); i++ ) {
                  Rule r = (Rule) rules.get(i); 
               
                  r.checkRule(flowContext.getHistory());  
                 }
             }
        }
    }

	/**
	 * Method that handles storing a snapshot copy of a <code>SessionHistory</code> instance of
	 * a workflow to a memory presistent store within the <code>WorkFlowWebContext</code> 
	 * @param historySnapshot the snapshot of the session history to store.
	 * @param workflowContext the current WorkFlowContxt.
	 */
	 protected void persistSnapshotToMemory(SessionHistory historySnapshot, WorkFlowContext workflowContext) {
		 
		 SessionHistory historySnapshotCopy = (SessionHistory) ObjectCopier.copy(historySnapshot);
		   workflowContext.storeWorkFlowSnaphot(workflowContext.getActiveView(), historySnapshotCopy); 
	 } 
}
