package org.jharks.workflow;


/**
 * The <CODE>Loader</CODE> class is an abstract class with a factory method that
 * allows for the instantiation of various loader implementations.  This 
 * ultimately allows for new <CODE>Loader</CODE> implementations to be written
 * and then plugged into the architecture by simply modifying the workflow 
 * XML configuration file. 
 * 
 * @author Michael J. Rizzo
 * @version 1.0
 */
public abstract class Loader {

    /** The alias name for the loader. */
    private String loaderAlias;
    
    /**
     * Performs load action.
     * @param rs the response set used to retrieve items to load.
     * @param sh a <code>SessionHistory</code> instance.
     * @param config the config object for the loader.
     */
    public abstract void load(ResponseSet rs, SessionHistory sh, 
            LoaderConfig config);
    
    /**
     * Performs load action.
     * @param rs the response set used to retrieve items to load.
     * @param sh a <code>SessionHistory</code> instance.
     * @param tr the transition object currently being processed.
     */
    public abstract void load(ResponseSet rs, SessionHistory sh, 
            Queueable tr);
    
    /**
     * Factory method for creation of Loader implementations.
     * @param loaderImplName the class name of loader implementation to be 
     *        instantiated.
     * @return an instance of a <CODE>Loader</CODE> object.
     * @throws ClassNotFoundException if can not find classname specified.
     * @throws IllegalAccessException if the protection on object constructor
     *         is violated.
     * @throws InstantiationException if error occurs while instantiating
     *         the object.
     */
    public static Loader getLoaderInstance(String loaderImplName) 
                                        throws ClassNotFoundException, 
                                               IllegalAccessException, 
                                               InstantiationException {
        
        return (Loader) Class.forName(loaderImplName).newInstance();     
    }

    /**
     * Returns the specified loader alias name. 
     * @return the loader alias.
     */
    public String getLoaderAlias() {
        return loaderAlias;
    }
    
    /**
     * Set the loader alias.
     * @param loaderAlias the loader alias name.
     */
    public void setLoaderAlias(String loaderAlias) {
        this.loaderAlias = loaderAlias;
    }
}
