package org.jharks.workflow;

import java.util.Iterator;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Configuration class for loader implementations. 
 * 
 * @author Michael J. Rizzo
 * @version 1.0
 */
public class LoaderConfig {

    /** The logger for the loader. */
    protected Log logger = LogFactory.getLog(getClass().getName());
    
    /** Represents type of nodes that will be loaded. */
    private String itemType;
    
    /** The question id of the response source. */
    private String questionId;
    
    /** The configured value to be placed on a node. */ 
    private String nodeValue;
    
    /** The list of <code>QueueFilter</code> instances. */
    private List<QueueFilter> filters;
    
    /**
     * Constructor that sets node type configuration. 
     * @param itemType the item type value to set.
     */
    public LoaderConfig(String itemType) {
        this.itemType = itemType;
    }
    
    /**
     * Constructor that sets node type and filters within configuration. 
     * @param itemType the item type value to set.
     * @param filters the collection of filters to add to configuration.
     */
    public LoaderConfig(String itemType, List<QueueFilter> filters) {
        this.itemType = itemType;
        this.filters = filters;
    }
    
    /**
     * Constructor that sets node type, question source id and a 
     * configured node value within configuration. 
     * @param itemType the item type value to set.
     * @param questionId the source question id to set.
     * @param nodeValue the configured node value.
     */
    public LoaderConfig(String itemType, String questionId, String nodeValue) {
        new LoaderConfig(itemType, questionId, nodeValue, null);
    }
    
    /**
     * Constructor that sets node type, question source id,  
     * configured node value and filters within configuration. 
     * @param itemType the item type value to set.
     * @param questionId the source question id to set.
     * @param nodeValue the configured node value.
     * @param filters the collection of filters to add to configuration.
     */
    public LoaderConfig(String itemType, String questionId, String nodeValue, List<QueueFilter> filters) {
        this.itemType = itemType;
        this.questionId = questionId;
        this.nodeValue = nodeValue;
        this.filters = filters;
    }
    
    
    /**
     * Returns the configured item node type.
     * @return the item node type.
     */
    public String getItemType() {
        return itemType;
    }
    
    /**
     * Set the item node type.
     * @param itemType The item type to set.
     */
    public void setItemType(String itemType) {
        this.itemType = itemType;
    }
    
    /**
     * Returns the source question id.
     * @return the source question id.
     */
    public String getQuestionId() {
        return questionId;
    }
    
    /**
     * Set the source question id.
     * @param questionId The source question id to set.
     */
    public void setQuestionId(String questionId) {
        this.questionId = questionId;
    }
    
    /**
     * Returns the configured node value.
     * @return the configured node value.
     */
    public String getNodeValue() {
        return nodeValue;
    }
    
    /**
     * Set the node value.
     * @param nodeValue The node value to set.
     */
    public void setNodeValue(String nodeValue) {
        this.nodeValue = nodeValue;
    }
    
    /**
     * Returns <code>iterator</code> of configured filters.
     * @return a iterator of the configured filters.
     */
    public Iterator<QueueFilter> filterIterator() {
       if (filters == null) 
           return null;
       
       return filters.iterator();
    }
    
    /**
     * Adds a filter to the configured filter collection.
     * @param filter the filter instance to add.
     */
    public void addFilters(List<QueueFilter> filter) {
        this.filters = filter;
    }
}
