package org.jharks.workflow;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;


/**
 * The ResponseSet object is used to encapsulate a set of one or more user 
 * responses to a portion of the survey.  Survey responses will be saved as a
 * name/value pair, in which the value will be a List of responses.  The name 
 * will correspond to the name of the question for which the responses 
 * were given.
 */
public class ResponseSet {

    /** The Map containing the responses. */
    private Map<String, List<Object>> responseMap = new LinkedHashMap<String, List<Object>>();
    
    /** The name of the question group associated with this response set. */
    private String questionGroup;
    
    /** Default public constructor. */
    public ResponseSet() {
    }
    
    /**
     * Adds the given response value to the response set for the given question name.
     * 
     * @param questionId the name of the question for the response.
     * @param value the value of the response to add.
     */
    public synchronized void addResponse(String questionId, Object value) {
        List<Object> responseList = responseMap.get(questionId);
        if (responseList == null) {
            responseList = new ArrayList<Object>();
            responseMap.put(questionId, responseList);
        }
        responseList.add(value);
    }
    
    /**
     * Adds the given response values to the response set for the given question name.
     * 
     * @param questionId the name of the question for the response.
     * @param values the values to add.
     */
    public void addResponses(String questionId, Collection<Object> values) {
        List<Object> responseList = responseMap.get(questionId);
        if (responseList == null) {
            responseList = new ArrayList<Object>();
            responseMap.put(questionId, responseList);
        }
        responseList.addAll(values);
    }
    
    /**
     * Adds the responses contained in another response set to this response 
     * set.
     * 
     * @param responses the responses to add.
     */
    public void addResponses(ResponseSet responses) {
        Iterator<String> responseIt = responses.responseMap.keySet().iterator();
        while (responseIt.hasNext()) {
            String key = (String) responseIt.next();
            addResponses(key, responses.getResponses(key));
        }
    }
    
    /**
     * Removes all of the responses that have been added to the 
     * response set for a question.
     * 
     * @param questionId the name of the question for which to remove the 
     *        responses.
     */
    public synchronized void removeResponses(String questionId) {
        responseMap.remove(questionId);
    }
    
    /**
     * Removes a single response that has been added to the response set for
     * a question.
     * 
     * @param questionId the name of the question for which to remove the 
     *        response.
     * @param value the response value to remove.
     */
    public synchronized void removeResponse(String questionId, String value) {
        List<Object> responseList = responseMap.get(questionId);
        if (responseList != null) {
            responseList.remove(value);
        }
    }
    
    /**
     * Returns the List of responses for a question.
     * 
     * @param questionId the name of the question for which to retrieve the 
     *        responses.
     * 
     * @return the responses for the question.
     */
    public List<Object> getResponses(String questionId) {
        return (List<Object>) responseMap.get(questionId);
    }
    
    /**
     * Returns a <code>Collection</code> containing the IDs of the questions
     * answered in this response set.
     * 
     * @return the IDs of the questions answered in this response set.
     */
    public Collection<String> getQuestionIds() {
        return responseMap.keySet();
    }
    
    /**
     * Returns the ResponseSet as a String.
     * 
     * @return the ResponseSet as a String.
     */
    public String toString() {
        return responseMap.toString();
    }

    /**
     * Set the question group for the response set.
     * 
     * @return the question group name.
     */
    public String getQuestionGroup() {
        return questionGroup;
    }

    /**
     * Sets the question group associated with this reponse set.
     * 
     * @param questionGroup the question group to set.
     */
    public void setQuestionGroup(String questionGroup) {
        this.questionGroup = questionGroup;
    }
    
    /**
     * Returns the sie of the key set that is present in the resposne set.
     * @return the size of the key set.
     */
    public int keySetSize() {
     return this.responseMap.size();   
    }
    
    /**
     * Retruns true if the key is present in the response set.
     * @param key the key to check for presence in the response set.
     * @return true if key present in the response set.
     */
    public boolean containsKey(Object key) {
        return this.responseMap.containsKey(key);
    }
}
