package org.jharks.workflow;

/**
 * The <CODE>RuleCondition</CODE> class is an abstract class with a factory
 * method that allows for the instantiation of various 
 * <CODE>RuleCondition</CODE> implementations.  <CODE>RuleCondition</CODE>s 
 * can be used as conditional evaluators on <CODE>Rule</CODE> implementations
 * This ultimately allows for new <CODE>RuleCondition</CODE> implementations 
 * to be written and then plugged into the architecture by simply modifying 
 * the workflow XML configuration file.
 * 
 * @author Michael J. Rizzo
 * @version 1.0
 */

public abstract class RuleCondition {

    /** Flag indicating whether condition should be applied using context. */ 
    private boolean useContext = false;
    
    /**
     * Check to see if condition applies according to provided input.
     * @param param the submitted input.
     * @return boolean indicating whether condition applied or not.
     */
    public abstract boolean checkCondition(Object param);
    
    /**
     * Factory method for creation of RuleCondition implementations.
     * @param rcImplName the class name of RuleCondition implementation
     *        to be instantiated.
     * @return an instance of a <CODE>Rule</CODE> object.
     * @throws ClassNotFoundException if can not find classname specified.
     * @throws IllegalAccessException if the protection on object constructor
     *         is violated.
     * @throws InstantiationException if error occurs while instantiating
     *         the object.
     */
    public static RuleCondition getRuleConditionInstance(String rcImplName) 
                                        throws ClassNotFoundException, 
                                               IllegalAccessException, 
                                               InstantiationException {
        
        return (RuleCondition) Class.forName(rcImplName).newInstance();     
    }
    
    /**
     * Returns true if condition should only look for responses
     * within the current answer context. Setting is false by default.
     * @return Returns true if context should be used to restrict condition.
     */
    public boolean isUseContext() {
        return useContext;
    }
    
    /**
     * Set whether to restrict to current context or not.
     * @param useContext The useContext to set.
     */
    public void setUseContext(boolean useContext) {
        this.useContext = useContext;
    }
}
