package org.jharks.workflow;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.EmptyStackException;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.collections.ArrayStack;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jharks.workflow.util.ContextInspector;
import org.jharks.workflow.util.Enumerator;


/**
 * The SessionHistory class maintains a history of the responses given during the 
 * current user interview.  This class is closely related to the ResponseSet class, 
 * since it also is used to hold name/value pairs of questions and their responses.
 * <br><br>
 * The additional methods on the SessionHistory object are used to inspect the responses.
 * 
 * @see ResponseSet
 */
public class SessionHistory implements Serializable {
    
	private static final long serialVersionUID = 1L;
	
    /** The logger for the module. */
    protected Log logger = LogFactory.getLog(getClass().getName());
    
    /** The Map containing the answer sets. */
    private Map<AnswerKey, LinkedHashSet<Answer>> responseMap = new LinkedHashMap<AnswerKey, LinkedHashSet<Answer>>();
    
    /** The stack for holding queued items. */
    private ArrayStack stack = new ArrayStack();
    
    /** The Map containing active session properties. */
    private HashMap<String, Object> properties = new HashMap<String, Object>();
    
    /** Identifier of persisted survey record. */
    private Long surveyId;
    
    /** Optional attribute which tracks. */
    private Integer groupId;
    
    /**
     * Adds the given response values to the response set for the given 
     * question name. Question group will be set to null.
     * 
     * @param questionId the name of the question for the response.
     * @param values the values to add.
     */
    public void addResponses(String questionId, Collection<Object> values) {
        addResponses(questionId, null, values);
    }
    
    /**
     * Adds the given response values to the response set for the given 
     * question name and question group.
     * 
     * @param questionId The name of the question for the response.
     * @param questionGroup The name of the question group.
     * @param values the values to add.
     */
    public void addResponses(String questionId, String questionGroup, Collection<Object> values) {
        AnswerKey key = new AnswerKey(questionId, questionGroup);
        
        addResponses(key, values);
    }
    
    /**
     * Adds the given response values to the response set for the given 
     * answer key.
     * 
     * @param key the name of the question for the response.
     * @param values the values to add.
     */
    public void addResponses(AnswerKey key, Collection<Object> values) {
        
        if ( responseMap.containsKey(key)) { 
            modifyAnswerSet(key, responseMap.get(key), values);
        }  else {
            addAnswerSet(key, values);
        }
    }
    
    /**
     * Adds the responses contained in another response set to this response set.
     * 
     * @param responses the responses to add.
     */
    public void addResponses(ResponseSet responses) {
        Iterator<String> responseIt = responses.getQuestionIds().iterator();
        while (responseIt.hasNext()) {
            String questionId = responseIt.next();
            AnswerKey key = new AnswerKey(questionId, 
                    responses.getQuestionGroup());
            addResponses(key, responses.getResponses(questionId));
        }
    }
    
    /**
     * Adds a pre-populated answer set to the history. The question group will
     * be set to null.
     * 
     * @param questionId the name of the question for the response.
     * @param answerSet a populated set that contains <CODE>Answer</CODE> 
     *        instances.
     */
    public void addAnswerSet(String questionId, LinkedHashSet<Answer> answerSet) {
        addAnswerSet(questionId, null, answerSet);
    }

    /**
     * Adds a pre-populated answer set to the history. 
     * 
     * @param questionId The name of the question for the response.
     * @param questionGroup The name of the question group.
     * @param answerSet a populated set that contains <CODE>Answer</CODE> 
     *        instances.
     */
    public void addAnswerSet(String questionId, String questionGroup, 
            LinkedHashSet<Answer> answerSet) {
        AnswerKey key = new AnswerKey(questionId, questionGroup);
        responseMap.put(key, answerSet);
    }
    
    /**
     * Returns the List of responses for a question. The question group in the
     * key must be null for a proper match.  This method does not use context.
     * 
     * @param questionName the name of the question for which to retrieve 
     *        the responses.
     * 
     * @return the responses for the question.
     */
    public List<Answer> getResponses(String questionName) {
        return getResponses(questionName, false);
    }
    
    /**
     * Returns the List of responses for a question. The method allows 
     * for filtering of reponses for the question by the current answer
     * context.  If the currentContext argument is set to "true"  then
     * only responses for the specified question that match the current
     * context will be returned. The question group must be null for a proper
     * match.
     * 
     * @param questionName The name of the question for which to retrieve
     * the responses.
     * @param useContext Flag indicating whether to filter question responses
     * by current answer context.
     * @return the responses matching the specified criteria.
     */
    public List<Answer> getResponses(String questionName, boolean useContext) {
       return getResponses(questionName, null, useContext); 
    }
    
    /**
     * Returns the List of responses for a question. The method allows 
     * for filtering of reponses for the question by the current answer
     * context.  If the currentContext argument is set to "true"  then
     * only responses for the specified question that match the current
     * context will be returned. 
     * 
     * @param questionName The name of the question for which to retrieve
     * the responses.
     * @param questionGroup The name of the question group.
     * @param useContext Flag indicating whether to filter question responses
     * by current answer context.
     * @return the responses matching the specified criteria.
     */
    public List<Answer> getResponses(String questionName, String questionGroup,
            boolean useContext) {
        AnswerKey key = new AnswerKey(questionName, questionGroup);
        if (useContext) 
            return getResponses(key, ContextInspector.retrieveContext(this));
        
        LinkedHashSet<Answer> answerSet = responseMap.get(key);
        List<Answer> responses = new ArrayList<Answer>();
        if (answerSet != null) responses.addAll(answerSet);
  
        return responses;
        
    }
    
    /**
     * Returns the List of <CODE>Answer<CODE> objects that contain the
     * specified property. If no matches are found an empty List is 
     * returned.
     * @param questionName The name of the question for which to retrieve the 
     *        responses.
     * @param property The property to filter responses by.
     * @return the responses for the question that contain the property in 
     *         their context.
     */
    public List<Answer> getResponses(String questionName, AnswerProperty property) {
        LinkedHashSet<Answer> answerSet = responseMap.get(
                new AnswerKey(questionName, null));
        List<Answer> responses = new ArrayList<Answer>();
        if (answerSet != null) {
          responses.addAll(getMatchingSubset(answerSet, property));
        }
        return responses;
    }
    
    /**
     * Returns the List of <CODE>Answer<CODE> objects that contain the
     * specified properties. If no matches are found an empty List is 
     * returned.
     * @param questionName The name of the question for which to retrieve the 
     *        responses.
     * @param properties The properties to filter responses by.
     * @return the responses for the question that contain the property in 
     *         their context.
     */
    public List<Answer> getResponses(String questionName, Set<AnswerProperty> properties) {
        LinkedHashSet<Answer> answerSet = responseMap.get(
                new AnswerKey(questionName, null));
        List<Answer> responses = new ArrayList<Answer>();
        if (properties != null) {
          Iterator<AnswerProperty> it = properties.iterator();
          Set<Answer> responseSet = new LinkedHashSet<Answer>();
          while (it.hasNext()) {
            AnswerProperty prop = it.next();
            if (responseSet.isEmpty()) {
                responseSet.addAll(getMatchingSubset(answerSet, prop));
            } else {
                responseSet.retainAll(getMatchingSubset(answerSet, prop));
            }
          }
          responses.addAll(responseSet);
        }
        return responses;
    }
    
    /** 
     * Returns the set of response keys in the response map. 
     * @return a set view of the response map keys in the response map.
     */
    public Set<AnswerKey> getResponseKeys() {
        return responseMap.keySet();
    }
    
    
    /**
     * Returns a subset of the passed set based on the specified property.
     * The property must exists in the answer context for the object to
     * be included in the returned subset.  
     * @param answerSet The answer set from which to filter out unmatching
     *        responses.
     * @param prop The property to filter by.
     * @return the subset of reponses that match the specified property.
     */
    private Set<Answer> getMatchingSubset(Set<Answer> answerSet, AnswerProperty prop) {
        
        Set<Answer> responseSet = new LinkedHashSet<Answer>(); 
          for (Iterator<Answer> it=answerSet.iterator(); it.hasNext(); ) {
            Answer answer = (Answer) it.next();
            if (answer.containsProperty(prop))
                responseSet.add(answer);
   
          }
        return responseSet;
    }
    
    /** 
     * Returns the List of <CODE>Answer<CODE> objects that match
     * the passed context.  The question group on the answer key must be
     * null for a roper match.
     * @param questionName The name of the question for which to retrieve the 
     *        responses.
     * @param context The specifed answer context.
     * @return the subset of reponses that match the specified context.
     */
    public List<Answer> getResponses(String questionName, AnswerContext context) {
        return getResponses(questionName, null, context);
    }
    
    /** 
     * Returns the List of <CODE>Answer<CODE> objects that match
     * the passed context.
     * @param questionName The name of the question for which to retrieve the 
     *        responses.
     * @param questionGroup The name of the question group.
     * @param context The specifed answer context.
     * @return the subset of reponses that match the specified context.
     */
    public List<Answer> getResponses(String questionName, String questionGroup, AnswerContext context) {
        AnswerKey key = new AnswerKey(questionName, questionGroup);
        
        return getResponses(key, context);
    }
    
    /** 
     * Returns the List of <CODE>Answer<CODE> objects that match
     * the passed context.
     * @param key The answer key to retrieve answers from the response map. 
     * @param context The specifed answer context.
     * @return the subset of reponses that match the specified context.
     */
    public List<Answer> getResponses(AnswerKey key, AnswerContext context) {
        LinkedHashSet<Answer> answerSet = responseMap.get(key);
        List<Answer> responses = new ArrayList<Answer>();
        
        if (answerSet != null) {
            for (Iterator<Answer> it=answerSet.iterator(); it.hasNext(); ) {
                Answer answer = it.next();
                
                if (context != null) {
                  if (answer.getContext().equals(context)) 
                     responses.add(answer);
                } else {
                    responses.add(answer);
                }
            }
        }
        
        return responses;
    }
    
    
    /**
     * Returns a single response to the question, or null if the question has no 
     * responses.  If the question has multiple responses, this method will return 
     * the first item in the list of responses.  The answer group must be null 
     * for a proper match and the method does not use context.
     * 
     * @param questionName the name of the question for which to return the value.
     * 
     * @return a single response for the question.
     */
    public Answer getSingleResponse(String questionName) {
       return getSingleResponse(questionName, null);
    }
    
    /**
     * Returns a single response to the question, or null if the question has no 
     * responses.  If the question has multiple responses, this method will return 
     * the first item in the list of responses.  The method does not use context.
     * 
     * @param questionName The name of the question for which to return the value.
     * @param questionGroup The group of the question.
     * 
     * @return a single response for the question.
     */
    public Answer getSingleResponse(String questionName, String questionGroup) {
        List<Answer> responseList = getResponses(questionName, questionGroup, false);
        if (responseList != null && responseList.size() > 0) {
            return responseList.get(0);
        } else {
            return null;
        } 
    }
    
    /**
     * Returns the number of responses the user submitted for the question. The 
     * answer group must be null for a proper match and the method does 
     * not use context.
     * 
     * @param questionName the name of the question for which to check the responses.
     * 
     * @return the number of responses the user submitted for the question.
     */
    public int getResponseCount(String questionName) {
        List<Answer> responseList = getResponses(questionName);
        return (responseList != null ? responseList.size() : 0);
    }
    
    /**
     * Remove response for the specified question.
     * 
     * @param questionName the name of the question for which to remove responses.
     * @param useContext Flag indicating whether to use current context when 
     * removing responses.
     */
    public void removeAnswers(String questionName, boolean useContext) {
        if (useContext)
            removeAnswers(questionName, ContextInspector.retrieveContext(this));
        
            removeAnswers(questionName);
     }
     
     
     /**
      * Remove response for the specified question, regardless of context.
      * @param questionName  the name of the question for which to remove responses.
      */
     private void removeAnswers(String questionName) {
         removeAnswers(questionName, null, null);
     }
     
     /**
      * Remove response for the specified question, regardless of context.
      * @param questionName The name of the question for which to remove 
      *        responses.
      * @param questionGroup The name of the question group.
      */
     public void removeAnswers(String questionName, String questionGroup) {
         removeAnswers(questionName, questionGroup, null);
     }
     
     /**
      * Remove responses for the specified question, regardless of context.
      * @param questionName  The name of the question for which to remove responses.
      * @param questionGroup The name of the question group.
      * @param context The provided answer context.
      */
     private void removeAnswers(String questionName, String questionGroup, 
             AnswerContext context) {
         removeAnswers(new AnswerKey(questionName, questionGroup), context);
     }
     
     /**
      * Remove response for the specified question and answer context.
      * 
      * @param questionName the name of the question for which to remove responses.
      * @param context the provided answer context.
      */
      public void removeAnswers(String questionName, AnswerContext context) {
            removeAnswers(new AnswerKey(questionName, null), context);
      }
      
      /**
       * Remove response for the specified question and answer context.
       * 
       * @param key The answer key for which to remove responses.
       * @param context the provided answer context.
       */
       public void removeAnswers(AnswerKey key, AnswerContext context) {
           LinkedHashSet<Answer> answerSet = responseMap.get(key);
           
           if (answerSet != null) {
              if (context != null) {
                removeAnswers(key, answerSet, context); 
              } else {
                removeAnswers(answerSet);
              }
           }
       }
    
    /**
     * Pushes a stackable survey element onto the queue stack.
     * @param element the survey element to be added.
     * @return the element justed added to the stack.
     */
    public Object stackElement(Stackable element) {
        return stack.push(element);
    }
    
    /**
     * Returns an iterator of the stackable survey elements
     * in the stack. 
     * @return an iterator of the elements in the stack.
     */
    @SuppressWarnings("unchecked")
	public Iterator<Stackable> stackIterator() {
        return stack.listIterator();
    }
    
    /**
     * Return the number of stackable survey elements in the stack.
     * @return the number of elements in the stack.
     */
    public int stackSize() {
        return stack.size();
    }
    
    /**
     * Return true if the queue is currently empty. 
     * @return true if queue is empty.
     */
    public boolean isStackEmpty() {
        return stack.isEmpty();
    }
    
    /**
     * Method returns the top element off the queue without removing it.
     * @return the top element on the queue. 
     * @throws EmptyStackException if the queue is empty.
     */
    public Stackable peek() throws EmptyStackException {
        return (Stackable) stack.peek();
    }
    
    /**
     * Method returns the n'th element down from the top of the queue.
     * @param i the number of elements deep.
     * @return the n'th element on the queue, zero relative.
     * @throws EmptyStackException if the queue is empty.
     */
    public Stackable peek(int i) throws EmptyStackException {
        return (Stackable) stack.peek(i);
    }
    
    /**
     * Pops the top element off the stack and returns it.
     * @return the top element on the stack.
     * @throws EmptyStackException
     */
    public Stackable popStack() throws EmptyStackException {
        return (Stackable) stack.pop();
    }
    
    /**
     * Remove all elements from the stack.
     */
    public void emptyStack() {
        this.stack.clear();
    }
    
    /**
     * Set the session property to the specified value.
     *
     * @param name Name of the property to set.
     * @param value The associated property value.
     */
    public void setProperty(String name, Object value) {
    
        // Name cannot be null
        if (name == null)
            throw new IllegalArgumentException("property name can not be null.");

        // Null value is the same as removeAttribute()
        if (value == null) {
            removeProperty(name);
            return;
        }

        properties.put(name, value);
    }
    
    /**
     * Remove the specified session property if it exists.
     *
     * @param name Name of the proeprty to remove
     */
    public void removeProperty(String name) {
        properties.remove(name);
    }
    
    /**
     * Return the value to which the passed key is mapped in the properties map
     * or null if the map contains no mapping for this key.
     *
     * @param name Name of the session property to return.
     * @return the requested property.
     */
    public Object getProperty(String name) {
        Object prop = properties.get(name);

        if (prop!=null) {
          return(prop);
        }  else {
          return null; 
        }
    }

    /**
     * Return the names of all session properties currently set, or an
     * empty <code>Enumeration</code> if there are none.
     * @return the property names as an <code>Enumeration</code>. 
     */
    public Enumeration<String> getPropertyNames() {
        return new Enumerator(properties.keySet());
    }
    
    /**
     * Returns a <code>Map</code> of the active set properties.
     * @return a <code>Map</code> of set properties. 
     */
    public Map<String, Object> getProperties() {
      return this.properties;    
    }
    
    /**
     * Modifys an existing set.  The method removes any answers
     * with a matching <CODE>AnswerContext</CODE> and then adds 
     * new values.
     * @param set The set to be modified.
     * @param values The new value objects to be added to the set. 
     */
    private void modifyAnswerSet(AnswerKey key, LinkedHashSet<Answer> set, Collection<Object> values) {
        AnswerContext context = getContext();
        
        //removeAnswers(key, set, context);
        set.clear();
        populateAnswerSet(values, context, set);
       
    }
    
    /**
     * Add new set of <CODE>Answer</CODE> instances to the response
     * map.
     * @param questionId The question id which will be set as the key
     *        in the reponse map.
     * @param values The collection of values to be added.
     */
    private void addAnswerSet(AnswerKey key, Collection<Object> values) {
        AnswerContext context = getContext();
        LinkedHashSet<Answer> answerSet = new LinkedHashSet<Answer>(); 
        populateAnswerSet(values, context, answerSet);
        
        responseMap.put(key, answerSet);
    }
    
    /**
     * Remove all <CODE>Answer</CODE> instances from the specified
     * set that match the passed <CODE>AnswerContext</CODE>.
     * @param key The answer key.
     * @param answerSet The set to be inspected.
     * @param context The <CODE>AnswerContext</CODE> to be matched.
     */
    private synchronized void removeAnswers(AnswerKey key, LinkedHashSet<Answer> answerSet, 
            AnswerContext context) {
        
        LinkedHashSet<Answer> savedAnswerSet = responseMap.get(key);
        
        for (Iterator<Answer> it=answerSet.iterator(); it.hasNext(); ) {
            Answer answer = it.next(); 
            if (answer.getContext().equals(context)) {
                savedAnswerSet.remove(answer);
            }
         }
    }
    
    /**
     * Remove all <CODE>Answer</CODE> instances from the specified
     * regardless of <CODE>AnswerContext</CODE>.
     * @param answerSet The set to be inspected.
     */
    private synchronized void removeAnswers(LinkedHashSet<Answer> answerSet) {
        //TODO: investigate for concuurent modification issue.
        for (Iterator<Answer> it=answerSet.iterator(); it.hasNext(); ) { 
            Answer answer = it.next(); 
            answerSet.remove(answer);
         }
    }
    
    /**
     * Populate set with <CODE>Answer</CODE> objects. A single 
     * <CODE>Answer</CODE> object is added for each 
     * <CODE>Object</CODE> in the value collection.
     * @param values The collection of value objects to be added.
     * @param context The <CODE>AnswerContext</CODE> instance to
     *        be matched.
     * @param answerSet The set to which to add <CODE>Answer</CODE>
     *        objects.
     */
    private void populateAnswerSet(Collection<Object> values, AnswerContext context, 
            LinkedHashSet<Answer> answerSet) {
        for (Iterator<Object> it=values.iterator(); it.hasNext(); ) {
            answerSet.add(new Answer(context, it.next()));
        }
    }
    
    /**
     * Simple wrapper method that calls retirever method on utility class. 
     * @return the current context.
     */
    private AnswerContext getContext() { 
        return ContextInspector.retrieveContext(this);
    }
    
    /**
     * Return the surveyId
     * @return Returns the surveyId.
     */
    public Long getSurveyId() {
        return surveyId;
    }
    
    /**
     * Set the survey id.
     * @param surveyId The surveyId to set.
     */
    public void setSurveyId(Long surveyId) {
        this.surveyId = surveyId;
    }

    /**
     * Set the data group id.
     * 
     * @return the group id.
     */
    public Integer getGroupId() {
        return groupId;
    }

    /**
     * Set the data group id.
     * 
     * @param groupId the group Id to set.
     */
    public void setGroupId(Integer groupId) {
        this.groupId = groupId;
    }
    
    /** Clears all current responses from the session history. */
    public void clearResponses() {
        responseMap.clear();   
    }
    
    /**
     * Removes the specified question and all it's response from the history object.
     * @param questionName The question to remove.
     * @param questionGroup The group context of the question.
     */
    public void removeQuestionAndResponses(String questionName, String questionGroup) {
        AnswerKey key = new AnswerKey(questionName, questionGroup);
        
        responseMap.remove(key);
    }
}
