package org.jharks.workflow;

import java.util.List;
import java.util.Set;


/**
 * The <CODE>org.jharks.workflow.State</CODE> interface contains
 * the methods to be implemented by the <CODE>StateImpl</CODE> implementation.  
 * 
 * @author Michael J. Rizzo
 * @version 1.0
 */

public interface State {

    /**
     * Method that will set the properties associated with a <CODE>StateImpl</CODE>
     * object.
     * @param props the <CODE>Set</CODE> of properties.
     */
    public void setProperties(Set<WorkFlowProperty> props);
    
    /** 
     * Method that will return the <CODE>Set</CODE> of properties associated 
     * with a <CODE>StateImpl</CODE> object.
     * @return the set properties.
     */
    public Set<WorkFlowProperty> getProperties();

    /**
     * Simple setter method that sets the name attribute of 
     * a <CODE>StateImpl</CODE> object.
     * @param name the name to set.
     */
    public void setName(String name);
    
    /** 
     * Simple getter method that retrieves the name attribute set on a 
     * <CODE>StateImpl</CODE> object.
     * @return the name attribute.
     */
    public String getName();
    
    /**
     * Set the view name.
     * @param viewName the name of the view.
     */
    public void setViewName(String viewName);
    
    /**
     * Retrieve the configured view for a <CODE>StateImpl</CODE> instance.
     * @return the name of the view.
     */
    public String getViewName();
    
    /**
     * Set the name of the filter classname for the view to use, if set.
     * @param viewFilterClassname the classname of the view filter.
     */
    public void setViewFilterClassname(String viewFilterClassname);
    
    /**
     * Retrieve the classname of the view filter if set.
     * @return the classnae of a configured view filter.
     */
    public String getViewFilterClassname();
    
    /**
     * Indicates whether the view controller should support back navigation
     * from this current state.
     * @return true if back navigation should be supported for this site.
     */
	public boolean isAllowBackNavigation();
    
	/**
	 * Set whether the state is being indicated to support back navigation.
	 * @param allowBackNavigation the flag indicator for whether back navigation should be supported.
	 */
	public void setAllowBackNavigation(boolean allowBackNavigation);
	
    /**
     * Set default next state.
     * @param stateKey the key label of the configured state.
     */
    public void setDefaultNextState(String stateKey);
    
    /**
     * Retrieve the default next state a <CODE>StateImpl</CODE> instance.
     * @return the key label of the configured default next state.
     */
    public String getDefaultNextState();
    
    /**
     * Set whether the state is an end state or not.
     * @param endFlag flag indicating whether state is an end state or not.
     */
    public void setEndState(boolean endFlag);
    
    /**
     * Retrieve the end state flag string.
     * @return the end state flag.
     */
    public boolean isEndState();
    
    /**
     * Add a <CODE>Transition</CODE> instance.
     * @param t the <CODE>Transition</CODE> to add.
     */
    public void addTransition(Transition t);
    
    /**
     * Method that will set the Transition objects available on a StateImpl object.
     * @param transitions List of transitions to be added.
     */
    public void setTransitions(List<Transition> transitions);
    
    /**
     * Method that allows for the retrieval of a <CODE>List</CODE> of 
     * <CODE>Transition</CODE> objects available on a <CODE>StateImpl</CODE> 
     * object.
     * @return a <CODE>List</CODE> of available transitions or null.
     */
    public List<Transition> getTransitions();
    
    /**
     * Method that allows for retrievel of <CODE>StateInitializer</CODE>.
     * Initializer will handle setup actions for state upon entry.
     * @return a <CODE>StateInitializer</CODE> or null.
     */
    public StateInitializer getInitializer();
    
    /**
     * Set a post processor for the <CODE>StateImpl</CODE> instance.
     * @param pp a <CODE>PostProcessor</CODE> instance.
     */
    public void setPostProcessor(PostProcessor pp);

    /**
     * Retrieves the configured <CODE>PostProcesor</CODE> object if one exists.
     * @return the <CODE>PostProcessor</CODE> instance for this object
     *         if exists or return null.
     */
    public PostProcessor getPostProcessor();
    
    /** 
     * Returns flag indicating whether state has been initialized or not.
     * @return true if state has been initialized.
     */
    public boolean isInitialized();

}
