package org.jharks.workflow;


import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


/**
 * The org.jharks.workflow.StateImpl class models a basic state 
 * object.  The state represents a finite point in a path.  In the case 
 * of ParentLink it represents the screen that a user is currently at 
 * within the survey.  The <CODE>StateImpl</CODE> object is associated with 
 * one to multiple <CODE>Transition</CODE> objects, which allow for movement
 * to another state.
 * 
 * @author Michael J. Rizzo
 * @version 1.0
 */

public class StateImpl implements State {

    /** The logger for the module. */
    protected Log logger = LogFactory.getLog(getClass().getName());
    
    /** The name of the <CODE>StateImpl</CODE> instance. */
    private String name;

    /** List of transition objects. */
    private List<Transition> transitions;
    
    /** The name of the associated view. */
    private String viewName;
    
    /** The classname of a specified view filter. */
    private String viewFilterClassname;
    
    /** The key label for a configured default state. */
    private String defaultNextState;
    
    /** The set of state properties */
    private Set<WorkFlowProperty> properties;
    
    /** The initializer object */
    private StateInitializer si; 
    
    /** The post processor object */
    private PostProcessor pp;
    
    private boolean initialized = false;
    
    /** Attribute indicating wether state is an end state or not. */
    private boolean endState = false;
    
    /** Attribute indicating whether back navigation from the state should be supported by the view controller. */
    private boolean allowBackNavigation = false;
    
    /** Constructor that initializes the transition list. */
    public StateImpl() {
        transitions = new ArrayList<Transition>();
    }
    
    /**
     * Simple getter method that retrieves the name attribute.
     * @return the name attribute. 
     */
    public String getName() {
        return name;
    }

    /**
     * Simple setter method that sets the name attribute.
     * @param name the String value to set as the name. 
     */
    public void setName(String name) {
        this.name = name;
    }

    /**
     * Set the object properties.
     * @param value the set of properties.
     */
    public void setProperties(Set<WorkFlowProperty> value) {
        this.properties = value;
    }

    /**
     * Add a property.
     * @param element the object to add to the property set.
     */
    public void addProperty(WorkFlowProperty element) {
        properties.add(element);
    }

    /**
     * Check if property set is empty.
     * @return true is the property set contains no elements.
     */
    public boolean isPropertySetEmpty() {
        return properties.isEmpty();
    }

    /**
     * Clear the property set.
     */
    public void clearPropertySet() {
        properties.clear();
    }

    /**
     * Method that returns the <CODE>Set</CODE> of properties associated with 
     * a <CODE>StateImpl</CODE> object.
     * @return the <CODE>Set</CODE> of properties.
     * @see org.jharks.workflow.State#getProperties()
     */
    public Set<WorkFlowProperty> getProperties() {
        return this.properties;
    }

    /**
     * Method that allows for the retrieval of a <CODE>List</CODE> of 
     * <CODE>Transition</CODE> objects available on a <CODE>StateImpl</CODE>
     * object.
     * @return the list of <CODE>Transition</CODE> instances.
     */
    public List<Transition> getTransitions() {
        return this.transitions;
    }

    /**
     * Sets the <CODE>Transition</CODE> objects available on 
     * a <CODE>StateImpl</CODE> object.
     * @param value a list of transition objects.
     */
    public void setTransitions(List<Transition> value) {
        this.transitions = value;
    }

    /**
     * Retrieve a <CODE>Transition</CODE> instance by an index.
     * @param i the index.
     * @return the <CODE>Transition</CODE> instance at the passed index.
     */
    public Transition getTransitions(int i) {
        return (Transition) transitions.get(i);
    }

    /**
     * Add a <CODE>Transition</CODE> object for the state.
     * @param tran the <CODE>Transition</CODE> instance to add.
     */
    public void addTransition(Transition tran) {
       transitions.add(tran);
    }
    
    /**
     * Set a initialization action for the <CODE>StateImpl</CODE> instance.
     * @param si a <CODE>StateInitializer</CODE> instance.
     */
    public void setInitializer(StateInitializer si) {
       if (logger.isDebugEnabled()) 
           logger.debug("Configuring StateInitializer for StateImpl: [" + 
                   this.getName() + "]");
       this.si = si;
    }
    
    /**
     * Retrieves the configured <CODE>StateInitializer</CODE> object if one exists.
     * @return the <CODE>StateInitializer</CODE> instance for this object
     *         if exists or return null.
     */
    public StateInitializer getInitializer() {
        return this.si;
    }
    
    /**
     * Set a post processor for the <CODE>StateImpl</CODE> instance.
     * @param pp a <CODE>PostProcessor</CODE> instance.
     */
    public void setPostProcessor(PostProcessor pp) {
       if (logger.isDebugEnabled()) 
           logger.debug("Configuring PostProcessor for StateImpl: [" + 
                   this.getName() + "]");
       this.pp = pp;
    }
    
    /**
     * Retrieves the configured <CODE>PostProcesor</CODE> object if one exists.
     * @return the <CODE>PostProcessor</CODE> instance for this object
     *         if exists or return null.
     */
    public PostProcessor getPostProcessor() {
        return this.pp;
    }

    /**
     * Returns the view name associated with the StateImpl.
     * @return Returns the viewName.
     */
    public String getViewName() {
        return this.viewName;
    }
    
    /**
     * Set the associated view name of the StateImpl.
     * @param viewName The viewName to set.
     */
    public void setViewName(String viewName) {
        this.viewName = viewName;
    }
    
    /**
     * Return the name of the view filter classname.
	 * @return the view filter classname.
	 */
	public String getViewFilterClassname() {
		return viewFilterClassname;
	}

	/**
	 * Set the view filter classname.
	 * @param viewFilterClassname the classname to set for a view filter.
	 */
	public void setViewFilterClassname(String viewFilterClassname) {
		this.viewFilterClassname = viewFilterClassname;
	}

	/**
     * Returns the default next state label associated with the StateImpl.
     * @return Returns the key label of the configured default next state
     *        or null.
     */
    public String getDefaultNextState() {
        return this.defaultNextState;
    }
    
    /**
     * Set the associated default next state label.
     * @param stateKey stateKey the key label of the configured state..
     */
    public void setDefaultNextState(String stateKey) {
        this.defaultNextState = stateKey;
    }
    
    /**
     * Return whether object is initialized or not.
     * @return Returns the initalized state.
     */
    public boolean isInitialized() {
        return initialized;
    }
    /**
     * Set initialized flag.
     * @param initialized The initalized state to set.
     */
    public void setInitialized(boolean initialized) {
        this.initialized = initialized;
    }

    /**
     * Returns indicator for whether state is an end state.
     * @return true if the state is an end state.
     */
    public boolean isEndState() {
        return endState;
    }

    /**
     * Sets whether the state is an end state. A state is not an end state
     * by default.
     * @param endState The flag indicator for whether end state or not.
     */
    public void setEndState(boolean endState) {
        this.endState = endState;
    }

    /**
     * Indicates whether the view controller should support back navigation
     * from this current state.
     * @return true if back navigation should be supported for this site.
     */
	public boolean isAllowBackNavigation() {
		return allowBackNavigation;
	}

	/**
	 * Set whether the state is being indicated to support back navigation.
	 * @param allowBackNavigation the flag indicator for whether back navigation should be supported.
	 */
	public void setAllowBackNavigation(boolean allowBackNavigation) {
		this.allowBackNavigation = allowBackNavigation;
	}
    
    
}
