package org.jharks.workflow;

import java.util.ArrayList;
import java.util.List;
import java.util.Iterator;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


/**
 * The org.jharks.workflow.StateInitializer class represents 
 * an initialization action for a state. 
 * 
 * @author Michael J. Rizzo
 * @version 1.0
 */

public class StateInitializer implements WorkFlowConstants{

    /** The logger for the module. */
    protected Log logger = LogFactory.getLog(getClass().getName()); 
    
    /** List of <CODE>Rule</CODE> instances configured. */
    private List<Rule> rules;
    
    /** List of <CODE>Action</CODE> instances configured. */
    private List<Action> actions;
    
    /** List of <CODE>Transition</CODE> instances configured. */
    private List<Transition> autoTransitions;
    
    /** Default public constructor that initiates rule list. */
    public StateInitializer() {
        rules = new ArrayList<Rule>();
        actions = new ArrayList<Action>();
        autoTransitions = new ArrayList<Transition>();
    }

    /**
     * Add a <CODE>Rule</CODE> object for the transition.
     * @param className the classname of the rule implementation.
     */
    public void addRule(String className) 
                         throws ClassNotFoundException, 
                         IllegalAccessException, 
                         InstantiationException { 
        
      rules.add( Rule.getRuleInstance(className) );
    }
    
    /**
     * Add an <CODE>Action</CODE> object for the transition.
     * @param className the classname of the action implementation.
     */
    public void addAction(String className) 
                         throws ClassNotFoundException, 
                         IllegalAccessException, 
                         InstantiationException { 
      
      actions.add( Action.getActionInstance(className) );
    }
    
    /**
     * Add a <CODE>Rule</CODE> object for the transition.
     * @param r <CODE>Rule</CODE> object to add.
     */
    public void addRule(Rule r) {
        rules.add(r);
    }
    
    /**
     * The method allows for the retrieval of the entire collection of 
     * <CODE>Rule</CODE> instances on a <CODE>Transition</CODE> object.
     * @return List of <CODE>Rule</CODE> instances.
     */
    public List<Rule> getRules() {
        return rules;
    }

    /**
     * Create rule list from pre-existing list.
     * @param value List of <CODE>Rule</CODE> instances to add.
     */
    public void setRules(List<Rule> value) {
        this.rules = value;
    }

    /**
     * Retrieve <CODE>Rule</CODE> from collection by index position.
     * @return the value <CODE>Object</CODE>. 
     */
    public Object getRule(int i) {
        return (Object) rules.get(i);
    }

    /**
     * Retrieve iterator for rules.
     * @return rules iterator.
     */
    public Iterator<Rule> rulesIterator() {
        return rules.iterator();
    }

    /**
     * Check if rules collection is empty.
     * @return true if rules collection contains no elements.
     */
    public boolean isRulesEmpty() {
        return rules.isEmpty();
    }

    /**
     * Removes all <CODE>Rule</CODE> instances from the rules collection.
     */
    public void clearRules() {
        rules.clear();
    }

    /**
     * Returns number of elements in the rules list.
     * @return the number of <CODE>Rule</CODE> instances in the rules list.
     */
    public int rulesSize() {
        return rules.size();
    }

    /**
     * Returns an array containing all the elements of the rules list.
     * @return an array containing the <CODE>Rule</CODE> instances 
     *         on the this <CODE>Transition</CODE> instance.
     */
    public java.lang.Object[] rulesToArray() {
        return (java.lang.Object[]) rules.toArray(new java.lang.Object[rules
            .size()]);
    }
    
    /**
     * Add an <CODE>Action</CODE> object for the transition.
     * @param action <CODE>Action</CODE> object to add.
     */
    public void addAction(Action action) {
        actions.add(action);
    }
    
    /**
     * The method allows for the retrieval of the entire collection of 
     * <CODE>Action</CODE> instances on a initializer.
     * @return List of <CODE>Action</CODE> instances.
     */
    public List<Action> getActions() {
        return actions;
    }
    
    /**
     * Method that allows for the retrieval of a <CODE>List</CODE> of 
     * <CODE>Transition</CODE> objects available on a <CODE>StateImpl</CODE>
     * object.
     * @return the list of <CODE>Transition</CODE> instances.
     */
    public List<Transition> getTransitions() {
        return this.autoTransitions;
    }

    /**
     * Sets the <CODE>Transition</CODE> objects available on 
     * a <CODE>StateImpl</CODE> object.
     * @param value a list of transition objects.
     */
    public void setTransitions(List<Transition> value) {
        this.autoTransitions = value;
    }

    /**
     * Add a <CODE>Transition</CODE> object for the state.
     * @param tran the <CODE>Transition</CODE> instance to add.
     */
    public void addAutoTransition(AutoTransition tran) {
       autoTransitions.add(tran);
    }
}
