package org.jharks.workflow;

import java.util.ArrayList;
import java.util.List;
import java.util.Iterator;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


/**
 * The org.jharks.workflow.StateTransition class adds queuable properties
 * to a transition.
 * 
 * @author Michael J. Rizzo
 */

public class StateTransition extends Transition implements Queueable, 
  Cleanable, WorkFlowConstants {

    /** The logger for the module. */
    protected Log logger = LogFactory.getLog(getClass().getName());
    
    /** Flag indicating whether related survey state should be persisited 
     *  when transition is applied. 
     */
    private boolean persist = false;
    
    /** Flag indicating whether input should be queued for later processing. */
    private boolean queueInput = false; 
    
    /** Indicates type of input */
    private String inputProperty = null;
    
    /** The value of the configured input */
    private String inputValue = null;
    
    /** Specifies the grouping to queue elements by. */ 
    private String queueBy = null;
    
    /** The question id within the response set to queue responses for.  Acts
     * as a filter. TODO: consider rework to allow for filter 
     * (collection of ids).
     */
    private String queueQuestionId;
    
    /** Flag indicating whether the transition is a clean up transition. */
    private boolean cleanUp = false;  
    
    /** List of <CODE>QueueFilter</CODE> instances configured. */
    private List<QueueFilter> filters;
    
    /** A set of configured <CODE>Rule</CODE> instances. */
    private RuleSet set;
    
    /** Default public constructor that initiates rule list. */
    public StateTransition() {
        super();
    }
   
    /**
     * Returns the configured <CODE>RuleSet</CODE>.
     * @return a set of <CODE>Rule</CODE> instances.
     */
    public RuleSet getRuleSet() {
        return set;
    }

    /**
     * Set the <CODE>RuleSet</CODE> object. 
     * @param set the <CODE>RuleSet</CODE> object.
     */
    public void setRuleSet(RuleSet set) {
        this.set = set;
    }
    
    /**
     * Add a <CODE>QueueFilter</CODE> object for the transition.
     * @param vf <CODE>QueueFilter</CODE> object to add.
     */
    public void addFilter(QueueFilter vf) {
        if (filters == null) filters = new ArrayList<QueueFilter>();
        filters.add(vf);
    }
    
    /**
     * Retrieve iterator for filters.
     * @return filter iterator.
     */
    public Iterator<QueueFilter> filterIterator() {
        return filters.iterator();
    }
    
    /**
     * The method allows for the retrieval of the entire collection of 
     * <CODE>QueueFilter</CODE> instances on a <CODE>Transition</CODE> object.
     * @return List of <CODE>Rule</CODE> instances.
     */
    public List<QueueFilter> getFilters() {
        return filters;
    }
    
    /**
     * Retrieve the persist flag string.
     * @return the persist flag.
     */
    public boolean isPersist() {
        return this.persist;
    }

    /**
     * Set the persist flag for the transition.
     * @param flag the flag indicating whether to perisit state 
     *        within a transition.
     */
    public void setPersist(boolean flag) {
        this.persist = flag;
    }
    
    /**
     * Retrieve the queue flag.
     * @return the queue flag.
     */
    public boolean isQueueInput() {
        return this.queueInput;
    }

    /**
     * Set the queue input flag for the transition.
     * @param flag the flag indicating whether to queue input 
     *        within a transition.
     */
    public void setQueueInput(boolean flag) {
        this.queueInput = flag;
    }
    
    /**
     * Retrieve the clean up flag.
     * @return the clean up flag.
     */
    public boolean isCleanUp() {
        return this.cleanUp;
    }

    /**
     * Set the clean up flag for the transition.
     * @param flag the flag indicating whether the tranisition is a clean up 
     *        transition.
     */
    public void setCleanUp(boolean flag) {
        this.cleanUp = flag;
    }
    
    /**
     * Returns input property configured for queuable items.
     * @return Returns the input property configured.
     */
    public String getInputProperty() {
        return inputProperty;
    }
    
    /**
     * Set the input property for queuing.
     * @param property the input property.
     */
    public void setInputProperty(String property) {
        this.inputProperty = property;
    }
    
    /**
     * Returns input value configured for the queuable item.
     * @return Returns the configured input value.
     */
    public String getInputValue() {
        return inputValue;
    }
    
    /**
     * Set the input type value for queued object.
     * @param value the input value.
     */
    public void setInputValue(String value) {
        this.inputValue = value;
    }

     /**
      * Returns the queue by alais.
      * @return the queue by alias.
      */
     public String getQueueBy() {
         return queueBy;
     }
    
    /**
     * Set the grouping to use when loading elements onto the queue.
     * @param queueBy The queueBy to set.
     */
    public void setQueueBy(String queueBy) {
        if (logger.isDebugEnabled())
            logger.debug(" Setting queue by: " + queueBy);
        
        this.queueBy = queueBy;
    }
    /**
     * Return the question id that indicates which question responses
     * within the <CODE>ResponseSet</CODE> should be queued. 
     * @return Returns the queueQuestionId.
     */
    public String getQueueQuestionId() {
        return queueQuestionId;
    }
    
    /**
     * Set the question id filter for queuing responses.
     * @param queueQuestionId The queueQuestionId to set.
     */
    public void setQueueQuestionId(String queueQuestionId) {
        this.queueQuestionId = queueQuestionId;
    }
}
