package org.jharks.workflow;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * The org.jharks.workflow.Transition class represents the link
 * between two states within the overall system.  Multiple Transition objects 
 * can be associated with a single <CODE>StateImpl</CODE> object, as the 
 * application flow will have configurable branch points.  Branch points being 
 * explicit points within the workflow where state transitions will be 
 * determined by the answers to particular questions.
 * 
 * @author Michael J. Rizzo
 */
public abstract class Transition {

    /** The logger for the module. */
    protected Log logger = LogFactory.getLog(getClass().getName());
    
    /** Name of transition. */
    private String name;
    
    /** The destination state to which to transition. */
    private String nextState;
    
    /** List of <CODE>Rule</CODE> instances configured. */
    private List<Rule> rules;
    
    /** The post processor object */
    private PostProcessor pp;
    
    /**
     * Default public constructor.
     */
    public Transition() {
        rules = new ArrayList<Rule>();
    }

    /**
     * Returns the name of the transition.
     * @return Returns the name.
     */
    public String getName() {
        return name;
    }
    
    /**
     * Set the name of the transition.
     * @param name The name to set.
     */
    public void setName(String name) {
        this.name = name;
    }
    
    /**
     * Returns the configured destination state.
     * @return the destination state.  
     */
    protected String getNextState() {
        return nextState;
    }

    /**
     * Sets the destination state. 
     * @param nextState the string representing the destination state key.
     */
    public void setNextState(String nextState) {
        this.nextState = nextState;
    }
    
    /**
     * Add a <CODE>Rule</CODE> object for the transition.
     * @param className the classname of the rule implementation.
     */
    public void addRule(String className) 
                         throws ClassNotFoundException, 
                         IllegalAccessException, 
                         InstantiationException { 
        
      rules.add( Rule.getRuleInstance(className) );
    }
    
    /**
     * Add a <CODE>Rule</CODE> object for the transition.
     * @param r <CODE>Rule</CODE> object to add.
     */
    public void addRule(Rule r) {
        rules.add(r);
    }
    
    /**
     * The method allows for the retrieval of the entire collection of 
     * <CODE>Rule</CODE> instances on a <CODE>Transition</CODE> object.
     * @return List of <CODE>Rule</CODE> instances.
     */
    public List<Rule> getRules() {
        return rules;
    }

    /**
     * Create rule list from pre-existing list.
     * @param value List of <CODE>Rule</CODE> instances to add.
     */
    public void setRules(List<Rule> value) {
        this.rules = value;
    }

    /**
     * Retrieve <CODE>Rule</CODE> from collection by index position.
     * @return the value <CODE>Object</CODE>. 
     */
    public Object getRule(int i) {
        return (Object) rules.get(i);
    }
    
    /**
     * Retrieve iterator for rules.
     * @return rules iterator.
     */
    public Iterator<Rule> rulesIterator() {
        return rules.iterator();
    }

    /**
     * Check if rules collection is empty.
     * @return true if rules collection contains no elements.
     */
    public boolean isRulesEmpty() {
        return rules.isEmpty();
    }

    /**
     * Removes all <CODE>Rule</CODE> instances from the rules colelction.
     */
    public void clearRules() {
        rules.clear();
    }

    /**
     * Returns number of elements in the rules list.
     * @return the number of <CODE>Rule</CODE> instances in the rules list.
     */
    public int rulesSize() {
        return rules.size();
    }

    /**
     * Returns an array containing all the elements of the rules list.
     * @return an array containing the <CODE>Rule</CODE> instances 
     *         on the this <CODE>Transition</CODE> instance.
     */
    public java.lang.Object[] rulesToArray() {
        return (java.lang.Object[]) rules.toArray(new java.lang.Object[rules
            .size()]);
    }
    
    /**
     * Set a post processor for the <CODE>StateImpl</CODE> instance.
     * @param pp a <CODE>PostProcessor</CODE> instance.
     */
    public void setPostProcessor(PostProcessor pp) {
       if (logger.isDebugEnabled()) 
           logger.debug("Configuring PostProcessor for Transition: [" + 
                   this.getName() + "]");
       this.pp = pp;
    }
    
    /**
     * Retrieves the configured <CODE>PostProcesor</CODE> object if one exists.
     * @return the <CODE>PostProcessor</CODE> instance for this object
     *         if exists or return null.
     */
    public PostProcessor getPostProcessor() {
        return this.pp;
    }
    
}
