package org.jharks.workflow;

import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;

import org.jharks.workflow.util.Enumerator;


/**
 * The ViewMapping class is used to specify a view that should be displayed
 * by the GUI component of the utilizing application.
 */
public class ViewMapping {
    
    /** The Map containing active session properties. */
    private HashMap<String, Object> viewProperties = new HashMap<String, Object>();
    
    /** The name of the view to display. */
    private String viewName;
    
    /** The indocator for whether the view should supply a back navigation feature. */
    private boolean allowBackNavigation = false;
    
    /**
     * ViewMapping constructor that takes name of view. The allow back navigation
     * will be false if this constructor is used.
     * @param viewName the name of the view to which to buil out by the view controller.
     */
    public ViewMapping(String viewName) {
        this.viewName = viewName;
    }
    
    /**
     * ViewMapping constructor that takes the name of the view and sets the allow back 
     * navigation attribute.
     * @param viewName the name of the view to build out by view controller.
     * @param allowBackNavigation 
     */
    public ViewMapping(String viewName, boolean allowBackNavigation) {
    	this.viewName = viewName;
    	this.allowBackNavigation = allowBackNavigation;
    }

    /**
     * Returns the name of the view to display.
     * 
     * @return Returns the viewName.
     */
    public String getViewName() {
        return viewName;
    }
    
    /**
     * Returns boolean indicator whether back navigation should be provided
     * by view controller.
     * @return true if view controller should support back navigation.
     */
    public boolean isBackNavigationAllowed() {
    	return this.allowBackNavigation;
    }
    
    /**
     * Set properties to be available on this mapping object.
     * @param properties the <code>Map</code> of properties to 
     *        be added to view properties.
     */
    protected void setViewProperties(Map<String, Object> properties) {
        this.viewProperties.putAll(properties);
    }
    
    public void setViewProperty(String propertyName, Object property) {
        this.viewProperties.put(propertyName, property);
    }
    
    /**
     * Return the value to which the passed key is mapped in the properties map
     * or null if the map contains no mapping for this key.
     *
     * @param name Name of the session property to return.
     * @return the requested property.
     */
    public Object getViewProperty(String name) {
        Object prop = viewProperties.get(name);

        if (prop!=null) {
          return(prop);
        }  else {
          return null; 
        }
    }
    
    /**
     * Return the names of all view properties available on this object, 
     * or an empty <code>Enumeration</code> if there are none.
     * @return the property names as an <code>Enumeration</code>. 
     */
    @SuppressWarnings("unchecked")
	public Enumeration<String> getViewPropertyNames() {
    	return new Enumerator(viewProperties.keySet());
    }
}
