package org.jharks.workflow;

import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;
import java.util.MissingResourceException;
import java.util.PropertyResourceBundle;
import java.util.ResourceBundle;



/**
 * Contains configuration information for a <code>WorkFlowManager</code>
 * instance.
 * 
 * @author Michael J. Rizzo
 */
public class WorkFlowConfig implements WorkFlowConstants {
    
    /** The resource bundle containing the application configurations. */
    private ResourceBundle bundle;
    
    /** A Map containing all of the configurations that have been loaded. */
    private static Map<String, WorkFlowConfig> configs = new HashMap<String, WorkFlowConfig>();
    
    /**
     * Private constructor used internally to create a 
     * <code>WorkFlowConfig</code> based on an configuration file.
     * 
     * @param fileName the name of the configuration file, which must be located in
     *        the application's classpath.
     */
    private WorkFlowConfig(String fileName) {
        this.bundle = ResourceBundle.getBundle(fileName);
    }
    
    /**
     * Constructor to build a a <code>WorkFlowConfig</code> instance
     * based on a property stream.
     * @param stream property file stream to read from.
     * @throws IOException if error occurs reading stream.
     */
    private WorkFlowConfig(InputStream stream) throws IOException {
        this.bundle = new PropertyResourceBundle(stream);
    }

    /**
     * Create a <code>WorkFlowConfig</code> instance using the specified
     * property file.
     * @param fileName the name of the proeprty file.
     * @return the newly created <code>WorkFlowConfig</code> instance.
     */
    public static synchronized WorkFlowConfig getInstance(String fileName) {
        WorkFlowConfig config = (WorkFlowConfig) configs.get(fileName);
        if (config == null) {
            config = new WorkFlowConfig(fileName);
            configs.put(fileName, config);
        }
        return config;
    }
    
    /**
     * Create a <code>WorkFlowConfig</code> instance using the default
     * property file name.
     * @return the newly created <code>WorkFlowConfig</code> instance.
     */
    public static synchronized WorkFlowConfig getInstance() {
        return getInstance(PROP_FILE_NAME);
    }
    
    /**
     * Create a <code>WorkFlowConfig</code> instance using the specified
     * property file stream.  If a stream has been loaded previously
     * with the same alias the passed stream will be ignored.
     * @param fileName the alias to reference the file stream by.
     * @param stream property file stream to read from.
     * @return the newly created <code>WorkFlowConfig</code> instance.
     * @throws IOException if error occurs reading input stream.
     */
    public static synchronized WorkFlowConfig getInstance(String fileName,
            InputStream stream) throws IOException {
        WorkFlowConfig config = (WorkFlowConfig) configs.get(fileName);
        if (config == null) {
            config = new WorkFlowConfig(stream);
            configs.put(fileName, config);
        }
        return config;
    }
    
    /**
     * Returns the configuration property with the given name.
     * 
     * @param name the name of the configuration property.
     * 
     * @return the value of the configuration property.
     */
    public String getProperty(String name) {
        try {
            return bundle.getString(name);
        } catch (MissingResourceException mre) {
            return null;
        }
    }
}
