package org.jharks.workflow;

import java.util.HashMap;


/**
 * Object representing the current workflow context.
 * 
 * @author Michael J. Rizzo
 *
 */
public class WorkFlowContext {
    
    /** Map that allows for persistence of scoped history objects. */ 
    protected HashMap<String, SessionHistory> scopeHistoryMap;
	
    /** Map that tracks snapsots of SessionHistory for workFlow, used to support backward web navigations. */
	private HashMap<String, SessionHistory> sessionHistorySnapshots = new HashMap<String, SessionHistory>();
    
    /** Tracks the current active state within the workflow. */
	private State activeState;
    
    /** A key identifying the current workflow type. */
    private WorkFlowKey workFlowKey;
	
    /** Indicator for whether autoPersist should be on. */
    private boolean autoPersist = false;
    
    /** Indicator for whether <code>SessionHistory</code> snapshots 
     * for a workfow will be saved, support backwards navigation.
     */ 
    private boolean workFlowHistorySnapshot = false;
    
    /** 
     * Public default constructor that initializes scoped map and session 
     * history scoped object.
     */
	public WorkFlowContext() {
        scopeHistoryMap = new HashMap<String, SessionHistory>();
        
        SessionHistory sessionHistory = new SessionHistory(); 
        	this.scopeHistoryMap.put(WorkFlowConstants.SESSION_SCOPE, sessionHistory);
	}
	
    /**
     * Constructor that allows for passing of history object that will session
     * scoped.
     * @param history The <code>History</code> object to set at session scope.
     */
	public WorkFlowContext(SessionHistory history) {
        scopeHistoryMap.put(WorkFlowConstants.SESSION_SCOPE, history); 
	}
    
    /**
     * Constructor that allows for passing of history object that will 
     * be scoped according to scope parameter.
     * @param history The <code>History</code> object to set at session scope.
     */
    protected WorkFlowContext(SessionHistory history, String scope) {
        scopeHistoryMap.put(scope, history); 
    }

	/**
     * Return the session scoped history.
     * 
	 * @return the history
	 */
	public SessionHistory getHistory() {
		return (SessionHistory) 
          scopeHistoryMap.get(WorkFlowConstants.SESSION_SCOPE);
	}

	/**
     * Set the session history scoped object.
     * 
	 * @param history The history to set.
	 */
	public void setHistory(SessionHistory history) {
        scopeHistoryMap.put(WorkFlowConstants.SESSION_SCOPE, history);
	}	
	
    /**
     * Return a specified scoped history.
     * 
     * @param scope The scope of the variable to return.
     * 
     * @return the specified scoped history object or null is specified scope
     *         is not present in Map. 
     */
    public SessionHistory getHistory(String scope) {
        return (SessionHistory) 
          scopeHistoryMap.get(scope);
    }

    /**
     * Set the history object at the specified scope.
     * 
     * @param history The history to set.
     * @param scope The scope at witch to set the history object.
     */
    public void setHistory(SessionHistory history, String scope) {
        scopeHistoryMap.put(scope, history);
    }   
    
	 /**
     * Retrieve the active state instance.
     * @return the active state instance. 
     */
    public State getActiveState() {
        return activeState;
    }

    /**
     * Set the active state.
     * @param activeState the state to set as active.
     */
    protected void setActiveState(State activeState) {
        this.activeState = activeState;
    }
    
    /**
     * Returns the currently active view according to the workflow component.
     * @return the name of the currently active view or null if an there is no
     *         current active state.
     */
    public String getActiveView() {
        if (this.activeState != null) 
            return this.activeState.getViewName();
        
        return null;
    }
    
    /**
     * Return whether the view controller should build out back navigation for
     * the currently active view within the workflow context.  Returns false
     * if there is not an active view and state within the context. 
     * @return true is backwards navigation should be built out by view controller.
     */
    public boolean isBackNavAllowedForActiveView() {
    	if (this.activeState != null)
    		return this.activeState.isAllowBackNavigation();
    	
    	return false;
    }
    
    /**
     * Return whether the view controller should apply a view filter on the view.
     * Returns null if there was not view controller specified in the workflow 
     * configuration for the view.  The actual functionality of the view filter
     * is comtrolled by the view controller being used.
     * @return
     */
    public String getFilterClassnameForActiveView() {
    	if (this.activeState != null)
    		 return this.activeState.getViewFilterClassname();
    	
    	return null;
    }

    /**
     * The work flow key representing the register implementation.
     * 
     * @return the work flow key.
     */
    public WorkFlowKey getWorkFlowKey() {
        return workFlowKey;
    }

    /**
     * Set the key for the workflow implementation.
     * 
     * @param workFlowKey The work flow key to set.
     */
    public void setWorkFlowKey(WorkFlowKey workFlowKey) {
        this.workFlowKey = workFlowKey;
    }

    /**
     * Check whether auto persist is currently set for workflow context.
     * @return true if auto persist is on.
     */
	public boolean isAutoPersist() {
		return autoPersist;
	}

	/**
	 * Change the auto persist indicator on the workflow context.
	 * @param autoPersist boolean parameter to turn on or off auto persist indicator.
	 */
	public void setAutoPersist(boolean autoPersist) {
		this.autoPersist = autoPersist;
	}

	/**
	 * Check whether the workflow history snapshot mode is on. 
	 * @return true if history snaspshots whould be stored in memory for the workflow.
	 */
	public boolean isWorkFlowHistorySnapshots() {
		return workFlowHistorySnapshot;
	}

	/**
	 * Change the workflow history snapshot mode.
	 * @param workFlowHistorySnapshot boolean parameter to turn on or off workflow history snapshot mode.
	 */
	public void setWorkFlowHistorySnapshot(boolean workFlowHistorySnapshot) {
		this.workFlowHistorySnapshot = workFlowHistorySnapshot;
	}
	
	/**
	 * Method used to add a workflow <code>SessionHistory</code> snapshot to the snapshot memory store.
	 * @param viewName the view name to be used as the key within the snapshot memory store.
	 * @param snapshot the snapshot <code>SessionHistory</code> instance to store.
	 */
	public void storeWorkFlowSnaphot(String viewName, SessionHistory snapshot) {
		this.sessionHistorySnapshots.put(viewName, snapshot);
	}
	
	/**
	 * Return the snapshot <code>SessionHistory</code> from the memory persistent store.
	 * @param viewName the view name, key for the store to return a saved snapshot.
	 * @return
	 */
	public SessionHistory getSnapshotFromStore(String viewName) {
		return this.sessionHistorySnapshots.get(viewName);
	}
	

}
