package org.jharks.workflow;

import java.io.InputStream;

import org.jharks.workflow.exception.ConfigurationException;
import org.jharks.workflow.exception.WorkFlowException;
import org.xml.sax.EntityResolver;


/**
 * The <CODE>org.jharks.workflow.WorkFlowManager</CODE> interface contains
 * the methods to be implemented by a workflow manager implementation.  
 * 
 * @author Michael J. Rizzo
 * @version 1.0
 */
public interface WorkFlowManager {

    /**
     * Method to perform default initialization of the <code>WorkFlowManager</code>.
     * The default initialization and whether a default workflow is initializedis deferred to the implementing class. 
     * @throws ConfigurationException thrown if erroring reading 
     *         workflow configuration.
     */
    public void init() throws ConfigurationException;
    
    /**
     * Method to configure workflow states of the module.  
     * 
     * @throws ConfigurationException thrown if erroring reading 
     *         workflow configuration.
     */
    public void init(String workflowDefinitionFileName) throws ConfigurationException;
    
    /**
     * Method to configure ststes of the module from a stream.
     * 
     * @param workflowDefinitionName the name be associated with the passed stream.
     * @param workflowDefinitionStream the workflow definition as a stream.
     * @throws ConfigurationException if issue occurss processing workflow definition stream.
     */
    public void init(String workflowDefinitionName, InputStream workflowDefinitionStream) throws ConfigurationException;
    
    /**
     * Sets a custom entity resolver to be used when initializing workflows.
     * Setting the custom resolver will only have an impact on the workflow
     * manager if set before initializing an instance. Generally workflow
     * managers are singleton objects maintained by one of the workflow 
     * registry implementations.
     * @param resolver a custom entity resolver instance.
     */
    public void setCustomEntityResolver(EntityResolver resolver);
    
    /**
     * Return the curom entity resolver instance or null if one was not set.
     * @return the curom entity resolver instance or null.
     */
    public EntityResolver getCustomEntityResolver();
    
    /**
     * Starts the configured workflow at the defined start
     * state.
     * @throws <code>WorkFlowException</code> if error occurs while trying to
     *         determine next view mapping.
     * @return The start state configured for this workflow.
     */
    public ViewMapping start(WorkFlowContext flowContext) 
      throws WorkFlowException;
    
    /**
     * Set a start state.
     * @param value the string representation of the start state
     *        name;
     * @param initialize the parameter that specifies whether initialization
     *        should be performed or not.
     * @throws a <code>ConfigurationException</code> if error occurs 
     *         initializing specified start state.
     */
    public void setStartState(String value, boolean initialize, 
            WorkFlowContext flowContext) throws ConfigurationException;
    
    /**
     * Set a start state.
     * @param value the string representation of the start state
     *        name;
     */
    public void setStartState(String value);
    
    /**
     * The getNextMapping method is called by a view controller 
     * in order to receive the next view in the application.  The 
     * <CODE>ViewMapping</CODE> represents a mapping object to be used by the
     * a view controller to determine the next layout or control to 
     * display within the UI.  The getNextView method implementation should
     * call the BaseWorkFlowManagerImpl's internal process method. 
     * 
     * @param input the <CODE>ResponseSet</CODE> containing submitted input to 
     *              be processed.
     * @throws <code>WorkFlowException</code> if error occurs while trying to
     *         determine next view mapping.
     * @return The determined <CODE>ViewMapping</CODE> to the caller.
     */
    public ViewMapping getNextMapping(ResponseSet input, 
            WorkFlowContext flowContext) throws WorkFlowException;
    
    /**
     * Method used to add a state object to workflow.
     * @param st the <CODE>StateImpl</CODE> instance.
     */
    public void addState(StateImpl st);
    
    /**
     * Clear all states previously populated.  
     */
    public void clearStates();
    
    /**
     * Method to add a <code>Loader</code> object to workflow loader map.
     * The key for the loader is the alias that was configured for the loader
     * in the workflow.xml file.
     * @param l the <CODE>Loader</CODE> instance.
     */
    public void addLoader(Loader l);
    
    
}
