package org.jharks.workflow;

import java.util.HashMap;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jharks.workflow.exception.ConfigurationException;
import org.jharks.workflow.exception.WorkFlowException;
import org.xml.sax.EntityResolver;


/**
 * Registry class that handles instantiation and maintenance of 
 * <CODE>BaseWorkFlowManagerImpl</CODE> objects.   
 *  
 * @author Michael J. Rizzo
 * @version 1.0
 */
public class WorkFlowRegistry {

    /** The singleton instance of the registry */
    public static WorkFlowRegistry REGISTRY = new WorkFlowRegistry();
  
    /** The logger for the module registry. */
    private static Log logger = LogFactory.getLog(
            WorkFlowRegistry.class.getName());
  
    /** 
     * Map for registering singleton <CODE>BaseWorkFlowManagerImpl</CODE> 
     * objects. 
     **/
    private static HashMap<String, Object> map = new HashMap<String, Object>();

    /** 
     * Protected constructor to prevent erroneous instantiantion.
     */
    protected WorkFlowRegistry() {
    }
    
    /**
     * Method determines the type of <CODE>BaseWorkFlowManagerImpl</CODE> to 
     * instantiate through introspection, and is based on the passed parameter.   
     *
     * @param classname Classname of <CODE>BaseWorkFlowManagerImpl</CODE> to be 
     *                  instantiated.
     * @return Object newly instantiated singleton object.
     */
    public static synchronized Object getInstance(String classname) throws WorkFlowException {    
       return getInstance(classname, null);    
    }

    /**
     * Method determines the type of <CODE>BaseWorkFlowManagerImpl</CODE> to 
     * instantiate through introspection, and is based on the passed parameter.   
     *
     * @param classname Classname of <CODE>BaseWorkFlowManagerImpl</CODE> to be 
     *                  instantiated.
     * @param resolver a custom entity resolver to be used  when initializing 
     *        the <code>WorkflowManager</code> instance. 
     * @return Object newly instantiated singleton object.
     */
    public static synchronized Object getInstance(String classname, 
            EntityResolver resolver) throws WorkFlowException {
    
        String cName = classname;
        
        Object workflowSingleton = map.get(cName);
        
        if( workflowSingleton != null ) {
         return workflowSingleton;
        }
        try {
          workflowSingleton = Class.forName(cName).newInstance();
          WorkFlowManager mgr = (WorkFlowManager) workflowSingleton;
          
          if (resolver != null) mgr.setCustomEntityResolver(resolver);
          
          mgr.init();
          if( logger.isInfoEnabled()) { 
            logger.info("Instantiated a new singleton: " + classname);
          }  
        } catch (ClassNotFoundException cnfe) {
           throw new WorkFlowException("WorkFlowRegistry Fatal Error: " +
                 "Can not find class to instantiate " + classname, cnfe);
        } catch (InstantiationException ie) {
           throw new WorkFlowException("WorkFlowRegistry Fatal Error: " +
                 "Can not instantiate the class " + classname, ie);
        } catch (IllegalAccessException ia) {
           throw new WorkFlowException("WorkFlowRegistry Fatal Error: " +
                 "Can not access class " + classname, ia);
        } catch (ConfigurationException cfge) {
           throw new WorkFlowException("ModuleRegistry Fatal Error: " +
                 "Can not configure BaseWorkFlowManagerImpl " + classname, cfge);
        } 
        map.put(cName, workflowSingleton);
        return workflowSingleton;
        
    }
        
}
