package org.jharks.workflow;

import java.io.InputStream;
import java.util.HashMap;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jharks.workflow.exception.ConfigurationException;
import org.jharks.workflow.exception.WorkFlowException;
import org.xml.sax.EntityResolver;


/**
 * Registry class that handles instantiation and maintenance of 
 * <CODE>WorkFlowManager</CODE> objects.  This registry store mappings of 
 * <CODE>WorkFlowManagers</CODE> based on predefined mappings types. 
 * 
 * @author Michael J. Rizzo
 * @version 1.0 
 *
 */
public class WorkFlowTypeRegistry {
    
    /** The singleton instance of the registry */
    public static WorkFlowTypeRegistry REGISTRY = new WorkFlowTypeRegistry();
    
    /** The logger for the module registry. */
    private static Log logger = LogFactory.getLog(
            WorkFlowTypeRegistry.class.getName());
    
    /** Map for registering singleton <CODE>WorkFlowManager</CODE> objects. */
    private static HashMap<WorkFlowKey, Object> map = 
        new HashMap<WorkFlowKey, Object>();
    
    /** 
     * Protected constructor to prevent erroneous instantiantion.
     */
    protected WorkFlowTypeRegistry() {
    }
    
    /**
     * Method determines the implmentation of <CODE>WorkFlowManager</CODE> to 
     * instantiate through introspection, and is based on the passed parameter.   
     *
     * @param key The <code>WorkFlowKey</code> of the singleton object.
     * @return Object newly instantiated singleton object.
     */
    public static synchronized Object getInstance(WorkFlowKey key) throws WorkFlowException { 
       
        return getInstance(key, "", null);
    }
    
    /**
     * Method determines the implmentation of <CODE>WorkFlowManager</CODE> to 
     * instantiate through introspection, and is based on the passed parameter.   
     *
     * @param key The <code>WorkFlowKey</code> of the singleton object.
     * @param xmlFile The xml workflow file for this 
     *                  <code>WorkFlowManager</code> instance.
     * @return Object newly instantiated singleton object.
     */
    public static synchronized Object getInstance(WorkFlowKey key, 
            String xmlFile) throws WorkFlowException { 
       
       return getInstance(key, xmlFile, null);
    }
    
    /**
     * Method determines the implmentation of <CODE>WorkFlowManager</CODE> to 
     * instantiate through introspection, and is based on the passed parameter.   
     *
     * @param key The <code>WorkFlowKey</code> of the singleton object.
     * @param xmlFile The xml workflow file for this 
     *                  <code>WorkFlowManager</code> instance.
     * @param resolver a custom entity resolver to be used  when initializing 
     *        the <code>WorkflowManager</code> instance. 
     * @return Object newly instantiated singleton object.
     */
    public static synchronized Object getInstance(WorkFlowKey key, 
            String xmlFile, EntityResolver resolver) throws WorkFlowException {
        
        if (key == null)
            throw new WorkFlowException("WorkFlowTypeRegistry configuration " +
                 "exception: Can not find defined workflow implementation");
        
        Object workflowSingleton = null;
        
        if(map.containsKey(key)) {
          workflowSingleton = map.get(key);
        
          if( workflowSingleton != null ) 
             return workflowSingleton;
        }
        
        String cName = key.getClassName();
        try {
          workflowSingleton = Class.forName(cName).newInstance();
          WorkFlowManager mgr = (WorkFlowManager) workflowSingleton;
          
          if (resolver != null) mgr.setCustomEntityResolver(resolver);
          
          if (xmlFile != null && !xmlFile.trim().equals("")) {
            mgr.init(xmlFile);
          } else {
            mgr.init();
          }
          
          if( logger.isInfoEnabled()) { 
            logger.info("Instantiated a new singleton: " + cName);
          }  
        } catch (ClassNotFoundException cnfe) {
           throw new WorkFlowException("WorkFlowRegistry Fatal Error: " +
                 "Can not find class to instantiate " + cName, cnfe);
        } catch (InstantiationException ie) {
           throw new WorkFlowException("WorkFlowRegistry Fatal Error: " +
                 "Can not instantiate the class " + cName, ie);
        } catch (IllegalAccessException ia) {
           throw new WorkFlowException("WorkFlowRegistry Fatal Error: " +
                 "Can not access class " + cName, ia);
        } catch (ConfigurationException cfge) {
           throw new WorkFlowException("ModuleRegistry Fatal Error: " +
                 "Can not configure BaseWorkFlowManagerImpl " + cName, cfge);
        } 
        map.put(key, workflowSingleton);
        
        return workflowSingleton;
        
    }
    
    /**
     * Method determines the implmentation of <CODE>WorkFlowManager</CODE> to 
     * instantiate through introspection, and is based on the passed parameter.   
     *
     * @param key The <code>WorkFlowKey</code> of the singleton object.
     * @param workflowDefinitionStream a stream that contains the workflow definiton 
     *                  content..
     * @param resolver a custom entity resolver to be used  when initializing 
     *        the <code>WorkflowManager</code> instance. 
     * @return Object newly instantiated singleton object.
     */
    public static synchronized Object getInstance(WorkFlowKey key, 
            InputStream workflowDefinitionStream, EntityResolver resolver) throws WorkFlowException {
        
        if (key == null)
            throw new WorkFlowException("WorkFlowTypeRegistry configuration " +
                 "exception: Can not find defined workflow implementation");
        
        Object workflowSingleton = null;
        
        if(map.containsKey(key)) {
          workflowSingleton = map.get(key);
        
          if( workflowSingleton != null ) 
             return workflowSingleton;
        }
        
        String cName = key.getClassName();
        try {
          workflowSingleton = Class.forName(cName).newInstance();
          WorkFlowManager mgr = (WorkFlowManager) workflowSingleton;
          
          if (resolver != null) mgr.setCustomEntityResolver(resolver);
          
          mgr.init(key.getType(), workflowDefinitionStream);
          
          if( logger.isInfoEnabled()) { 
            logger.info("Instantiated a new singleton: " + cName);
          }  
        } catch (ClassNotFoundException cnfe) {
           throw new WorkFlowException("WorkFlowRegistry Fatal Error: " +
                 "Can not find class to instantiate " + cName, cnfe);
        } catch (InstantiationException ie) {
           throw new WorkFlowException("WorkFlowRegistry Fatal Error: " +
                 "Can not instantiate the class " + cName, ie);
        } catch (IllegalAccessException ia) {
           throw new WorkFlowException("WorkFlowRegistry Fatal Error: " +
                 "Can not access class " + cName, ia);
        } catch (ConfigurationException cfge) {
           throw new WorkFlowException("ModuleRegistry Fatal Error: " +
                 "Can not configure BaseWorkFlowManagerImpl " + cName, cfge);
        } 
        map.put(key, workflowSingleton);
        
        return workflowSingleton;
        
    }
}
