package org.jharks.workflow.actions;

import java.util.Iterator;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jharks.workflow.Action;
import org.jharks.workflow.Answer;
import org.jharks.workflow.QueueNode;
import org.jharks.workflow.QueueWrapper;
import org.jharks.workflow.SessionHistory;
import org.jharks.workflow.WorkFlowContext;
import org.jharks.workflow.exception.WorkFlowActionException;


/**
 * An action used to populate a queue based on a question id.  Places a
 * node on the queue for each response to a question.
 * @author Michael J. Rizzo
 * @version 1.0
 */
public class PopulateQueueAction extends Action {

    /** The question id from which to retrieve responses. */
    private String questionId;
    
    /** Flag indicating whether to use answer context. */
    private boolean useContext = false;
    
    /** The name of the queue to populate. */
    private String queueName;
    
    /** The logger for the module. */
    protected Log logger = LogFactory.getLog(getClass().getName());
    
    /**
     * @see org.jharks.workflow.Action#apply(java.lang.Object)
     */
    public void apply(Object input) throws WorkFlowActionException {
        if (input instanceof WorkFlowContext) {
            WorkFlowContext workFlowContext = (WorkFlowContext) input;
        	SessionHistory sh = workFlowContext.getHistory();
        
            List<Answer> responses = sh.getResponses(getQuestionId(), isUseContext());
            if (responses != null & responses.size() > 0)
                populateQueue(sh, responses);
            
        } else {
            throw new ClassCastException("Expecting SessionHistory Object.");
        }
    }
    
    /**
     * Getter method for question id property.
     * @return Returns the questionId.
     */
    public String getQuestionId() {
        return questionId;
    }
    
    /**
     * Setter method for question id property. 
     * @param questionId The questionId to set.
     */
    public void setQuestionId(String questionId) {
        this.questionId = questionId;
    }
    
    /**
     * Returns true if condition should only look for responses
     * within the current answer context. Setting is false by default.
     * @return Returns true if context should be used to restrict condition.
     */
    public boolean isUseContext() {
        return useContext;
    }
    
    /**
     * Set whether to restrict to current context or not.
     * @param useContext The useContext to set.
     */
    public void setUseContext(boolean useContext) {
        this.useContext = useContext;
    }

    /**
     * @return Returns the queueName.
     */
    public String getQueueName() {
        return queueName;
    }
    
    /**
     * @param queueName The queueName to set.
     */
    public void setQueueName(String queueName) {
        this.queueName = queueName;
    }
    
    /**
     * Places nodes on the queue based on the responses to a question.
     * @param history a <code>SessionHistory</code> instance.
     * @param responses list of <code>Answers</code> instances.
     */
    protected void populateQueue(SessionHistory history, List<Answer> responses) {
     
        QueueWrapper sElement = null;
        boolean stack = false;
        String queueLabel = getQueueName(); 
        
        if (history.isStackEmpty() ||
                (!(history.peek().getElementType().equalsIgnoreCase(queueLabel)))) {
                sElement = new QueueWrapper(queueLabel);
                stack = true;
              } else {
                //Category name same as top queue on stack so add to queue.
                sElement = (QueueWrapper) history.peek();
              }  
        
        for (Iterator<Answer> itr=responses.iterator(); itr.hasNext(); ) {
          Answer answer = itr.next();     
          
          String value = determineValue(answer);
          
          QueueNode qn = new QueueNode(queueLabel, value);  
        
            if (logger.isDebugEnabled())
              logger.debug("QueueByResponse: ItemType = [" + queueLabel + "] " +
                      "Value = [" + value + "]");
              sElement.add(qn);
                  
          }
      
      //Add queue to stack 
      if ( stack ) history.stackElement(sElement);  
    
    }
    
    /**
     * Determines the value of the <code>Answer</code> instance.
     * @param answer an <code>Answer</code> instance.
     * @return the value of the <code>Answer</code> instance.
     */
    protected String determineValue(Answer answer) {
        return answer.getValue().toString();
    }
}
