package org.jharks.workflow.actions;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.jharks.workflow.Action;
import org.jharks.workflow.SessionHistory;
import org.jharks.workflow.WorkFlowContext;
import org.jharks.workflow.exception.WorkFlowActionException;


/**
 * Action used to remove answers for one or many answer sets within a particular
 * question group.
 *  
 * This class takes two required attributes that should be configured 
 * as &lt;property&gt; elements of the &lt;workflow-action&gt; tag:
 * <ul>
 * <li>questionGroup - the key of the question group to remove reponses for from 
 *                  the <code>SessionHistory</code>.</li>
 * <li>questionId - the string value to match use when identifying answers for a
 *                  particular question within the question group to be removed 
 *                  against.</li>
 * </ul>
 * 
 * Multiple &lt;property&rt; elements can be used to configure multiple 
 * questionIds to compare <code>answerKeys</code> against from removal from the
 * history object.  
 * 
 * Currently this action does not support removal of answers based on a <code>
 * AnswerContext</code>
 * 
 * @author Michael J. Rizzo
 * @version 1.0
 */
public class RemoveAnswerHistoryAction extends Action {

    /** The constant seperator used for values. */
    private final static String SEPARATOR = ";";
    
    /** The name of the proprety to set. */
    private String questionGroup;
    
    /** the list ofquestion ids for which to remove answers. */
    private List<String> questionId = new ArrayList<String>();
    
    /**
     * @see org.jharks.workflow.Action#apply(java.lang.Object)
     */
    public void apply(Object input)throws WorkFlowActionException {
      if (input instanceof WorkFlowContext) {
    	  WorkFlowContext workFlowContext = (WorkFlowContext) input;
          SessionHistory sh = workFlowContext.getHistory();
          
          if (logger.isDebugEnabled())
              logger.debug("Removing answers: " +
                    "question group name = " + getQuestionGroup());
          
          for (Iterator<String> it = questionId.iterator(); it.hasNext();) {
            String questionId = it.next();
            
            sh.removeQuestionAndResponses(questionId, questionGroup);
          }
      } else {
          throw new ClassCastException("Expecting SessionHistory Object.");
      }
    }

    /**
     * Returns the specified question group.
     * @return the question group name.
     */
    public String getQuestionGroup() {
        return questionGroup;
    }

    /**
     * Sets the question group name.
     * @param questionGroup the question group name to set.
     */
    public void setQuestionGroup(String questionGroup) {
        this.questionGroup = questionGroup;
    }
    
    /**
     * Getter method for value property. Returns all values
     * within the value list as a delimited string.
     * @return Returns the value string.
     */
    public String getQuestionId() {
        Object[] valArray = questionId.toArray();
        StringBuffer result = new StringBuffer();
        if (valArray.length > 0) {
            result.append(valArray[0]);
            for (int i=1; i<valArray.length; i++) {
                result.append(SEPARATOR);
                result.append(valArray[i]);
            }
        }
        return result.toString();
    }
    
    /**
     * Setter method for value property.
     * @param value The value to set.
     */
    public void setQuestionId(String questionId) {
        
        this.questionId.add(questionId);
        if( logger.isDebugEnabled() )
            logger.debug("RemoveAnswerHistoryAction: added question id to " +
                    "list = [" + questionId + "]: size now = [" 
                    + this.questionId.size() + "]");
    }
    

}
