package org.jharks.workflow.conditions;

import java.util.List;

import org.jharks.workflow.Answer;
import org.jharks.workflow.SessionHistory;


/**
 * The class will check whether any response exist for a dynamic
 * question.  A dynamic question is composed of configured question
 * id that is conditional upon a current property set within the 
 * <code>SessionHistory</code> object.  
 * 
 * This class takes two attributes that should be configured 
 * as &lt;property&gt; elements of the &lt;rule-condition&gt; tag:
 * <ul>
 * <li>questionId - the key of the question to check for any reponses
 *                  in the <code>SessionHistory</code>.</li>
 * <li>propertyName - the session property name to append to the dynamic 
 *                  questionId key.</li>
 * </ul>
 * 
 * If the property name is set the value currently set for the property in the
 * session will be appended to the questionId.  The session responseSet will 
 * then be interrogated for any reponses to the dynamic key (questionId +
 * property value). 
 * 
 * For example, to check whether any reponses exists for the medication question
 * when within a Problem Class processing loop you could use a transition 
 * configured as:
 * <pre>
 * &lt;transition name="moveAllergySpecial" next-state="alSp1"&gt;
 *  &lt;rule type="org.jharks.workflow.rules.GenericOrRule"&gt;
 *   &lt;rule-condition type="org.jharks.workflow.conditions
 *                           .DynaResponseExistCondition"&gt;
 *       &lt;property name="questionId" value="medQ1"/&gt;
 *       &lt;property name="propertyName" value="Problem Class"/&gt;
 *   &lt;/rule-condition&gt;
 *  &lt;/rule&gt;    
 * &lt;/transition&gt;
 * </pre>
 *   
 * @author Michael J. Rizzo
 * @version 1.0
 */
public class DynaResponseExistCondition extends ResponseExistCondition {

    /** The session property which will be used in building dynamic key */
    private String propertyName;
    
    /**
     * @see org.jharks.workflow.RuleCondition#checkCondition(
     *      java.lang.Object)
     */
    public boolean checkCondition(Object param) {
        if (param instanceof SessionHistory) {
            SessionHistory sh = (SessionHistory) param;
            
            StringBuffer sb = new StringBuffer();
            
            if(getQuestionId() != null) {
                sb.append(getQuestionId());
            } 
            
            if (getPropertyName() != null) {
               String prop = (String) sh.getProperty(getPropertyName()); 
               if (prop != null && (! prop.trim().equals(""))) 
                   sb.append("-")
                     .append(prop);
            }
            
            List<Answer> responses = sh.getResponses(sb.toString());
            if (responses != null && responses.size() > 0) {
                return true;
            }
            
            return false;
        } else {
            throw new ClassCastException("Expecting Session History Object.");
        }   
    }
    
    /**
     * Return the configured session property name.
     * @return Returns the propertyName.
     */
    public String getPropertyName() {
        return propertyName;
    }
    
    /**
     * Set the session property name to be used in buidling the dynamic 
     * question key.
     * @param propertyName The propertyName to set.
     */
    public void setPropertyName(String propertyName) {
        this.propertyName = propertyName;
    }
}
