package org.jharks.workflow.conditions;

import java.util.List;

import org.jharks.workflow.Answer;
import org.jharks.workflow.SessionHistory;



/**
 * Condition that checks if multiple responses exist for a particular question.
 * The limit of response can be set at configuration time.
 * 
 * This class takes two attributes that should be configured 
 * as a &lt;property&gt; element of the &lt;rule-condition&gt; tag:
 * <ul>
 * <li>questionId - the key of the question to check for any reponses
 *                  in the <code>SessionHistory</code>.</li>
 * <li>responseLimit - the number of responses to check for.  If the 
 *                  number of responses submitted is greater than the
 *                  responseLimit, the condition will return true. If
 *                  no limit is set by default it will return true if
 *                  there is more than one response. 
 * </ul>
 * 
 * For example, to check whether more than one reponses exists for the 
 * the selection of medication concentrations you could use a 
 * transition configured as:
 * <pre>
 * &lt;transition name="moveConcWarning" next-state="medConcWarning"&gt;
 *  &lt;rule type="org.jharks.workflow.rules.GenericOrRule"&gt;
 *   &lt;rule-condition type="org.jharks.workflow.conditions
 *                            .MultipleResponsesCondition""&gt;
 *       &lt;property name="questionId" value="medQ8"/&gt;
 *   &lt;/rule-condition&gt;
 *  &lt;/rule&gt;    
 * &lt;/transition&gt;
 * </pre>
 * 
 * @author Michael J. Rizzo
 * @version 1.0
 */
public class MultipleResponsesCondition extends ResponseExistCondition {
    
    /** The response limit for the question. */
    private int responseLimit = 1;
    
    /**
     * @see org.jharks.workflow.RuleCondition#checkCondition(
     *      java.lang.Object)
     */
    public boolean checkCondition(Object param) {
        if (param instanceof SessionHistory) {
            SessionHistory sh = (SessionHistory) param;
            
            if (logger.isDebugEnabled())
                logger.debug(this.getClass().getName() + 
                        ": using current context = " + isUseContext());
            
            List<Answer> responses = sh.getResponses(getQuestionId(), isUseContext());
            
            if (responses != null && responses.size() > getResponseLimit()) {
                if (logger.isDebugEnabled())
                  logger.debug("Multiple responses exist: size = [" 
                          + responses.size() + "] response limit = [" 
                          + getResponseLimit() + "]");
                return true;
            }
            
            return false;
        } else {
            throw new ClassCastException("Expecting Session History Object.");
        }   
    }
    
    /**
     * Return the response limit for the question.
     * @return Returns the response limit.
     */
    public int getResponseLimit() {
        return responseLimit;
    }
    
    /**
     * Set the response limit.
     * @param responseLimit The response limit to set.
     */
    public void setResponseLimit(int responseLimit) {
        this.responseLimit = responseLimit;
    }
}
