package org.jharks.workflow.conditions;

import java.util.ArrayList;
import java.util.List;

import org.jharks.workflow.Answer;
import org.jharks.workflow.SessionHistory;
import org.jharks.workflow.util.WorkFlowUtilities;


/**
 * The class will retrieve responses for the question id that was
 * declared in a &lt;property&gt; tag and attempt to match any of the response 
 * values against any of the specified values.
 * 
 * This class takes two required attributes that should be configured 
 * as &lt;property&gt; elements of the &lt;rule-condition&gt; tag:
 * <ul>
 * <li>questionId - the key of the question to retrieve reponses for from the
 *                  <code>SessionHistory</code>.</li>
 * <li>value - the string value to match reponses against.</li>
 * </ul>
 * 
 * Multiple &lt;property&rt; elements can be used to configure multiple values
 * to compare reponse values against.  If any of the reponse values match
 * any of the configured comparison values the condition will be met and return
 * true.  
 * 
 * For example, to check whether the symptom selected was either
 * <code>asthma</code> or <code>seizures</code> and move to the appropriate 
 * next view, you could use a transition configured as:
 * <pre>
 *  &lt;transition name="moveRespiratory" next-state="re0"&gt;
 *   &lt;rule type="org.jharks.workflow.rules.GenericOrRule"&gt;
 *    &lt;rule-condition type="org.jharks.workflow.conditions.
 *                             MultipleStringMatchCondition"&gt;
 *      &lt;property name="questionId" value="symQ1"/&gt;
 *      &lt;property name="value" value="asthma"/&gt;
 *      &lt;property name="value" value="seizures"/&gt;
 *     &lt;/rule-condition&gt;
 *    &lt;/rule&gt;    
 *   &lt;/transition&gt;
 * </pre>
 * 
 * @author Michael J. Rizzo
 * @version 1.0
 */
public class MultipleStringMatchCondition extends StringMatchCondition {

    /** The constant seperator used for values. */
    private final static String SEPARATOR = ";";
    
    /** A list containing possible values to match reponses against. */
    private List<String> value = new ArrayList<String>();
    
    /**
     * @see org.jharks.workflow.RuleCondition#checkCondition(
     *      java.lang.Object)
     */
      public boolean checkCondition(Object param) {
      boolean result = false;    
        if (logger.isDebugEnabled())
              logger.debug("MultipleStringMatchCondition: Check condition: " +
                    "questionId = " + this.getQuestionId());
              
        if (param instanceof SessionHistory) {
            SessionHistory rs = (SessionHistory) param;
            List<Answer> responses = rs.getResponses(this.getQuestionId());
                
            if (responses != null) {
              if (logger.isDebugEnabled())
                  logger.debug ("Size response set " + responses.size());
              
              for (int i=0; i < responses.size(); i++) {
                  result = compare(responses.get(i));
                  if (result) break;
              }
            }
        } else {
            throw new ClassCastException("Expecting SessionHistory Object.");
        }
      return result;
    }
    
    /**
     * Compare method to check equivalence.  Compares response values
     * against each string in the value list.
     * @param response the response for the configured question.
     * @return true if response matches one of configured values.
     */
    private boolean compare(Object response) {
        
        Object answerValue = WorkFlowUtilities.getAnswerValue(response); 
        
        boolean cmpResult = false;
        
        if (answerValue instanceof String) {
            
            String stResp = (String) answerValue;
            
            for (int i=0; i < value.size(); i++) {
                String val = value.get(i);
                
                if( logger.isDebugEnabled() )
                    logger.debug("MultipleStringMatchCondition: comparison " +
                      "value = [" + val + "]: response value = [" 
                      + stResp + "]");
                
                if (stResp.equalsIgnoreCase(val)) {
                  cmpResult = true;
                  break;
                }
            }      
            return cmpResult;
        } else {
            throw new ClassCastException("Expecting String Object.");
        }   
    }
    
    /**
     * Getter method for value property. Returns all values
     * within the value list as a delimited string.
     * @return Returns the value string.
     */
    public String getValue() {
        Object[] valArray = value.toArray();
        StringBuffer result = new StringBuffer();
        if (valArray.length > 0) {
            result.append(valArray[0]);
            for (int i=1; i<valArray.length; i++) {
                result.append(SEPARATOR);
                result.append(valArray[i]);
            }
        }
        return result.toString();
    }
    
    /**
     * Setter method for value property.
     * @param value The value to set.
     */
    public void setValue(String value) {
        
        this.value.add(value);
        if( logger.isDebugEnabled() )
            logger.debug("MultipleStringMatchCondition: added value to " +
                    "list = [" + value + "]: response size = [" 
                    + this.value.size() + "]");
    }
}
