package org.jharks.workflow.conditions;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jharks.workflow.RuleCondition;
import org.jharks.workflow.SessionHistory;


/**
 * The class will check to see if a session property is set and
 * check whether the property value matches the specified value.
 * 
 * This class takes two required attributes that should be configured 
 * as &lt;property&gt; elements of the &lt;rule-condition&gt; tag:
 * <ul>
 * <li>property - the name of the session property to check the configured
 *                value against.</li>
 * <li>value - the string value to match against the retrieved session
 *             property value.</li>
 * </ul>
 *   
 * For example, to check whether there is a <code>Issue</code> property set with a  
 * value of <code>skin_rash</code> you could use a transition configured as:
 * <pre>
 *  &lt;transition name="moveIssue" next-state="alSp0"&gt;
 *   &lt;rule type="org.jharks.workflow.rules.GenericOrRule"&gt;
 *    &lt;rule-condition type="org.jharks.workflow.conditions.
 *                             MultipleStringMatchCondition"&gt;
 *      &lt;property name="propertyName" value="Issue"/&gt;
 *      &lt;property name="value" value="slin_rash"/&gt;
 *     &lt;/rule-condition&gt;
 *    &lt;/rule&gt;    
 *   &lt;/transition&gt;
 * </pre>
 * 
 * @author Michael J. Rizzo
 * @version 1.0
 */
public class PropertyMatchCondition extends RuleCondition {

    /** The logger for the module. */
    protected Log logger = LogFactory.getLog(getClass().getName());
    
    /** The session property to be checked. */
    private String propertyName;
    
    /** The value of the session property. */
    private String value;
    
    /**
     * @see org.jharks.workflow.RuleCondition#checkCondition(
     * java.lang.Object)
     */
    public boolean checkCondition(Object param) {
         
        if (logger.isDebugEnabled())
              logger.debug("PropertyMatchCondition: Check condition: " +
                    "property = " + getPropertyName());
              
        if (param instanceof SessionHistory) {
            SessionHistory sh = (SessionHistory) param;
            Object property = sh.getProperty(this.getPropertyName());
            
            if (property == null)
                return false;
                
            if (value.equalsIgnoreCase(((String) property)))
                return true;
            
            return false;
        } else {
            throw new ClassCastException("Expecting SessionHistory Object.");
        }
    }

    /**
     * Return the configured session property name.
     * @return Returns the property.
     */
    public String getPropertyName() {
        return propertyName;
    }
    
    /**
     * Set the session property name.
     * @param propertyName The property name to set.
     */
    public void setPropertyName(String propertyName) {
        this.propertyName = propertyName;
    }
    
    /**
     * Getter method for value property.
     * @return Returns the value.
     */
    public String getValue() {
        return value;
    }
    /**
     * Setter method for value property.
     * @param value The value to set.
     */
    public void setValue(String value) {
        this.value = value;
    }
}
