package org.jharks.workflow.conditions;

import org.jharks.workflow.RuleCondition;
import org.jharks.workflow.SessionHistory;
import org.jharks.workflow.Stackable;

/**
 * This <CODE>RuleCondition</CODE> peeks into the stack of queues on
 * the <CODE>SessionHistory</CODE> object and checks to make sure
 * that the queue on the top of the stack matches the category
 * condition.
 * 
 * For example, to check whether the next queue on the stack is of type
 * <code>Problem Class</code> you could configure a <code>rule-condition</code>
 * as:
 * <pre>
 *    &lt;rule-condition type="org.jharks.workflow.conditions.
 *                             QueueTypeCondition"&gt;
 *      &lt;property name="category" value="Problem Class"/&gt;
 *    &lt;/rule-condition&gt;
 * </pre>
 * 
 * @author Michael J. Rizzo
 * @version 1.0
 */
public class QueueTypeCondition extends RuleCondition {

    /** The category type for object stored on queue. */
    private String category; 
    
    /**
     * @see org.jharks.workflow.RuleCondition#checkCondition(
     *      java.lang.Object)
     */
    public boolean checkCondition(Object param) {
        if (param instanceof SessionHistory) {
            SessionHistory sh = (SessionHistory) param;
            Stackable element = sh.peek(); 
            
            if (element.getElementType().equalsIgnoreCase(category))
              return true;  
            
            return false;   
        } else {
            throw new ClassCastException("Expecting SessionHistory Object.");
        }
    }
    
    /**
     * Returns the category of the stackable element configured for the 
     * condition.
     * @return the element category.
     */
    public String getCategory() {
        return category;
    }
    
    /**
     * Sets the category of the element.
     * @param category The element category.
     */
    public void setCategory(String category) {
        this.category = category;
    }

}
