package org.jharks.workflow.conditions;

import org.jharks.workflow.QueueNode;
import org.jharks.workflow.QueueWrapper;
import org.jharks.workflow.RuleCondition;
import org.jharks.workflow.SessionHistory;

/**
 * This <CODE>RuleCondition</CODE> peeks into the stack of queues
 * on the <CODE>SessionHistory</CODE> object and checks to make sure
 * that the queue on the top of the stack matches the category
 * condition and that the transitional property equals a specified
 * value.
 *
 * For example, to check whether the next queue on the stack is of type
 * <code>Problem Class</code> and with a transitional value of <code>
 * gastrointestinal</code> you could configure a <code>rule-condition</code>
 * as:
 * <pre>
 *    &lt;rule-condition type="org.jharks.workflow.conditions.
 *                             QueueTypeCondition"&gt;
 *      &lt;property name="category" value="Problem Class"/&gt;
 *      &lt;property name="value" value="gastrointestinal"/&gt;
 *    &lt;/rule-condition&gt;
 * </pre>
 * 
 * 
 * @author Michael J. Rizzo
 * @version 1.0
 */
public class QueueValueCondition extends RuleCondition {

    /** The name of the transitional property. */
    private String propertyName; 
    
    /** The value of the property. */
    private String value;
    
    /**
     * @see org.jharks.workflow.RuleCondition#checkCondition(
     *      java.lang.Object)
     */
    public boolean checkCondition(Object param) {
        if (param instanceof SessionHistory) {
            SessionHistory sh = (SessionHistory) param;
            QueueWrapper element = (QueueWrapper) sh.peek();
            
            if (element.getElementType().equalsIgnoreCase(propertyName)) {
                if (element.isEmpty()) 
                    return false;
                
                String responseValue = ((QueueNode) element.get()).getValue();
                  if (responseValue.equalsIgnoreCase(getValue()))
                    return true;  
            }
            
            return false;   
        } else {
            throw new ClassCastException("Expecting SessionHistory Object.");
        }
    }

    /**
     * Returns the name of the property.
     * @return the property name.
     */
    public String getPropertyName() {
        return propertyName;
    }
    
    /**
     * Sets the property name to evaluate.
     * @param property The property name.
     */
    public void setPropertyName(String property) {
        this.propertyName = property;
    }
    
    /**
     * Returns the value of the property.
     * @return the property value.
     */
    public String getValue() {
        return value;
    }
    
    /**
     * Sets the value of the property to evaluate.
     * @param value The property value.
     */
    public void setValue(String value) {
        this.value = value;
    }
}
