package org.jharks.workflow.conditions;

import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jharks.workflow.Answer;
import org.jharks.workflow.RuleCondition;
import org.jharks.workflow.SessionHistory;


/**
 * The class will check whether any response exist for a question.  
 * 
 * This class takes one attribute that should be configured 
 * as a &lt;property&gt; element of the &lt;rule-condition&gt; tag:
 * <ul>
 * <li>questionId - the key of the question to check for any reponses
 *                  in the <code>SessionHistory</code>.</li>
 * </ul>
 * 
 * For example, to check whether any reponses exists for the symptom question
 * you could use a transition configured as:
 * <pre>
 * &lt;transition name="moveSymptom" next-state="alSp1"&gt;
 *  &lt;rule type="org.jharks.workflow.rules.GenericOrRule"&gt;
 *   &lt;rule-condition type="org.jharks.workflow.conditions
 *                            .ResponseExistCondition""&gt;
 *       &lt;property name="questionId" value="symQ1"/&gt;
 *       &lt;property name="questionGroup" value="symptoms"/&gt;
 *   &lt;/rule-condition&gt;
 *  &lt;/rule&gt;    
 * &lt;/transition&gt;
 * </pre>
 * 
 * @author Michael J. Rizzo
 * @version 1.0
 */
public class ResponseExistCondition extends RuleCondition {

    /** The logger for the module. */
    protected Log logger = LogFactory.getLog(getClass().getName());
    
    /** The question identifier to which question applies */
    private String questionId;
    
    /** The question group identifier to which question applies */
    private String questionGroup;
    
    /**
     * @see org.jharks.workflow.RuleCondition#checkCondition(
     *      java.lang.Object)
     */
    public boolean checkCondition(Object param) {
        if (param instanceof SessionHistory) {
            SessionHistory sh = (SessionHistory) param;
            
            if (logger.isDebugEnabled())
                logger.debug(this.getClass().getName() + 
                        ": using current context = " + isUseContext());
            
            List<Answer> responses = sh.getResponses(getQuestionId(), 
                    getQuestionGroup(), isUseContext());
            if (responses != null && responses.size() > 0) {
                if (logger.isDebugEnabled())
                  logger.debug("Responses exist: size = " + responses.size());
                return true;
            }
            
            return false;
        } else {
            throw new ClassCastException("Expecting Session History Object.");
        }   
    }
    
    /**
     * Getter method for question id property.
     * @return Returns the questionId.
     */
    public String getQuestionId() {
        return questionId;
    }
    
    /**
     * Setter method for question id property. 
     * @param questionId The questionId to set.
     */
    public void setQuestionId(String questionId) {
        this.questionId = questionId;
    }
    
    /**
     * Getter method for question group property.
     * @return the question group.
     */
    public String getQuestionGroup() {
        return questionGroup;
    }

    /**
     * Setter method for the question group property.
     * @param questionGroup The questionGroup to set.
     */
    public void setQuestionGroup(String questionGroup) {
        this.questionGroup = questionGroup;
    }
}
