package org.jharks.workflow.conditions;

import java.util.Arrays;
import java.util.List;

import org.jharks.workflow.SessionHistory;

/**
 * Checks to see whether any of the values listed in the sessionPropertyValue attribute
 * of the rule condition is present in the session property indicated by sessionPropertyName attribute.
 * Multiple values are set using a comma delimited list.  If any (one to many ) of the values 
 * is present in the session property list value then the <code>RuleCondition</code> evaluates to 
 * true.  
 * <p>
 * Notes:<br>  
 * - If the session property object indicated by the sessionPropertyName attribute is not a list then the 
 * <code>RuleCondition</code> will throw a <code>ClassCastException</code>.
 * - The value match for this <code>RuleCondition</code> is case sensitive.
 * 
 * @author Michael J. Rizzo
 *
 */
public class SessionPropertyListMemberCondition extends SessionPropertyMatchCondition {
	
	/**
	 * {@inheritDoc}
     * @see org.jharks.workflow.RuleCondition#checkCondition(java.lang.Object)
     */
	@Override
	public boolean checkCondition(Object param) {
		
		if (logger.isDebugEnabled())
            logger.debug("SessionPropertyListMemberCondition: Check condition: " +
                  "property name = " + getSessionPropertyName());
        
		if (param instanceof SessionHistory) {
	          SessionHistory rs = (SessionHistory) param;
	  
	          if ( checkProperty(getSessionPropertyName(), rs) != null ) {
	              Object property = rs.getProperty(getSessionPropertyName());
	  
	            if (property instanceof List) {
	            	if (logger.isDebugEnabled()) {
	                    logger.debug("Session Property value = [" +  property.toString() + "]");
	                    logger.debug("Configured value = [" 
	                            + getSessionPropertyValue() + "]");
	                  }
	            	
	            	//Convert comma delimited list of values set in xml to list.
	            	List<String> propertyValues = Arrays.asList(getSessionPropertyValue().split("\\s*,\\s*"));
	            	for (String propertyValue : propertyValues) {
	            		//if any of the values listed in xml sessionPropertyValue attribute is present in 
	            		//the session property list object then evaluate to true.
						if (((List<?>) property).contains(propertyValue))
								return true;
					}
	            } else {
	        	  throw new ClassCastException("The " + this.getClass().getName() + 
	        			  " requires the sesion property value to be a list.");
	            }
	          }
		} else {
			throw new ClassCastException("Expecting SessionHistory Object.");
		}
		return false;
	}

}
