package org.jharks.workflow.conditions;

import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jharks.workflow.Answer;
import org.jharks.workflow.RuleCondition;
import org.jharks.workflow.SessionHistory;
import org.jharks.workflow.util.WorkFlowUtilities;


/**
 * Cap insensitive <CODE>String</CODE> match <CODE>RuleCondition</CODE>.
 * The class will retrieve responses for the question id that was
 * declared in a &lt;property&gt; tag and attempt to match any of the response 
 * values against the specified value.
 * 
 * This class takes two required attributes that should be configured 
 * as &lt;property&gt; elements of the &lt;rule-condition&gt; tag:
 * <ul>
 * <li>questionId - the key of the question to retrieve reponses for from the
 *                  <code>SessionHistory</code>.</li>
 * <li>value - the string value to match reponses against.</li>
 * </ul>
 * For example, to check whether the symptom selected was <code>asthma</code>
 * and move to the appropriate next view you could use a transition 
 * configured as:
 * <pre>
 *  &lt;transition name="moveRespiratory" next-state="re0"&gt;
 *   &lt;rule type="org.jharks.workflow.rules.GenericOrRule"&gt;
 *    &lt;rule-condition type="org.jharks.workflow.conditions.
 *                             StringMatchCondition"&gt;
 *      &lt;property name="questionId" value="symQ1"/&gt;
 *      &lt;property name="questionGroup" value="symptomForm"/&gt;
 *      &lt;property name="value" value="asthma"/&gt;
 *    &lt;/rule-condition&gt;
 *   &lt;/rule&gt;    
 *  &lt;/transition&gt;
 * </pre>
 *  
 * @author Michael J. Rizzo
 */
public class StringMatchCondition extends RuleCondition {

    /** The logger for the module. */
    protected Log logger = LogFactory.getLog(getClass().getName());
    
    /** The question identifier to which question applies */
    private String questionId;
    
    /** The question group identifier to which question applies */
    private String questionGroup;
    
    /** The value to which to match a response value. */ 
    private String value;
    
    /**
     * @see org.jharks.workflow.RuleCondition#checkCondition(
     *      java.lang.Object)
     */
    public boolean checkCondition(Object param) {
      boolean result = false;    
        if (logger.isDebugEnabled())
              logger.debug("MatchCondition: Check condition: " +
                    "questionId = " + getQuestionId());
              
        if (param instanceof SessionHistory) {
            SessionHistory rs = (SessionHistory) param;
            
            if (logger.isDebugEnabled())
                logger.debug(this.getClass().getName() + 
                        ": using current context = " + isUseContext());
            
            List<Answer> responses = rs.getResponses(getQuestionId(), 
                    getQuestionGroup(), isUseContext());
                
            if (responses != null) {
              if (logger.isDebugEnabled())
                  logger.debug ("Size response set " + responses.size());
              
              for (int i=0; i < responses.size(); i++) {
                  result = compare(responses.get(i));
                  if (result) break;
              }
            }
        } else {
            throw new ClassCastException("Expecting SessionHistory Object.");
        }
      return result;
    }
    
    /**
     * Compare method to check equivalence of strings.
     * @param response the response for the configured question.
     * @return true if response matches value property.
     */
    private boolean compare(Object response) {
       
       Object answerValue = WorkFlowUtilities.getAnswerValue(response); 
        
       if (answerValue instanceof String) {
           if( logger.isDebugEnabled() )
             logger.debug("StringMatchCondition: comparison value = [" 
               + this.value + "]: response value = [" 
               + (String) answerValue + "]");
             
           return (((String) answerValue).equalsIgnoreCase(this.value));
       } else {
           throw new ClassCastException("Expecting String Object.");
       }
    }
    
    /**
     * Getter method for question id property.
     * @return Returns the questionId.
     */
    public String getQuestionId() {
        return questionId;
    }
    
    /**
     * Setter method for question id property. 
     * @param questionId The questionId to set.
     */
    public void setQuestionId(String questionId) {
        this.questionId = questionId;
    }
    
    /**
     * Getter method for question group property.
     * @return the question group.
     */
    public String getQuestionGroup() {
        return questionGroup;
    }

    /**
     * Setter method for the question group property.
     * @param questionGroup The questionGroup to set.
     */
    public void setQuestionGroup(String questionGroup) {
        this.questionGroup = questionGroup;
    }
    
    /**
     * Getter method for value property.
     * @return Returns the value.
     */
    public String getValue() {
        return value;
    }
    
    /**
     * Setter method for value property.
     * @param value The value to set.
     */
    public void setValue(String value) {
        this.value = value;
    }

}
