package org.jharks.workflow.loaders;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jharks.workflow.Answer;
import org.jharks.workflow.LoaderConfig;
import org.jharks.workflow.QueueNode;
import org.jharks.workflow.QueueWrapper;
import org.jharks.workflow.Queueable;
import org.jharks.workflow.ResponseSet;
import org.jharks.workflow.SessionHistory;


/**
 * Queues up a response set by adding a node to the data structure for every 
 * single answer to a question within the response set. Answers can be omitted
 * by configuring queue-filter(s).
 * 
 * @author Michael J. Rizzo
 */
public class ResponseQueueLoader extends AbstractQueueLoader {

    /** The logger for the loader. */
    protected Log logger = LogFactory.getLog(getClass().getName());
    
    /**
     * Iterates over the question ids in the <CODE>ResponseSet</CODE> object
     * then over each reponse per question and adds an element to the queue 
     * for each response to each question id.  The stacked object represents the 
     * answer response. 
     * @param rs the <CODE>ResponseSet</CODE> to be processed.
     * @param sh a <code>SessionHistory</code> instance.
     * @param config a <code>LoaderConfig</code> instance.
     * 
     * @see org.jharks.workflow.Loader#load(ResponseSet, 
     *      SessionHistory, LoaderConfig)
     */
    public void load(ResponseSet rs, SessionHistory sh, LoaderConfig config) {

        Collection<String> questionIds = null;
        
        if(config.getQuestionId() == null) {
          questionIds = rs.getQuestionIds();
        } else {
          questionIds = new ArrayList<String>();
          questionIds.add(config.getQuestionId());
        }
        
        QueueWrapper sElement = null;
        boolean stack = false;
        String itemType = config.getItemType();
        
        if (sh.isStackEmpty() ||
            (!(sh.peek().getElementType().equalsIgnoreCase(itemType)))) {
            sElement = new QueueWrapper(itemType);
            stack = true;
        } else {
            //Category name same as top queue on stack so add to queue.
            sElement = (QueueWrapper) sh.peek();
        }
        
        for (Iterator<String> it=questionIds.iterator(); it.hasNext(); ) {
            String id = it.next();
            Collection<Object> responses = rs.getResponses(id); 
              for (Iterator<Object> itr=responses.iterator(); itr.hasNext(); ) {
                String responseValue = (String) itr.next();
                if (filter(responseValue, config)) continue;
                QueueNode qn = new QueueNode(itemType, responseValue);
                if (logger.isDebugEnabled())
                  logger.debug("QueueByResponse: ItemType = [" + itemType + "]");
                sElement.add(qn);
              }
        } 
        
        if (logger.isDebugEnabled())
            logger.debug("QueueSize = [" + sElement.queueSize() + "]");
            logger.debug("QueueName = [" + sElement.getElementType() + "]");
        
        //Add queue to stack 
        if ( stack && sElement.queueSize() > 0) sh.stackElement(sElement);
        
    }
    
    /**
     * Iterates over the question ids in the <CODE>ResponseSet</CODE> object
     * then over each reponse per question and adds an element to the queue 
     * for each response to each question id.  The stacked object represents the 
     * answer response. 
     * @param rs the <CODE>ResponseSet</CODE> to be processed.
     * @param sh a <code>SessionHistory</code> instance.
     * @param tr a queueable <code>Transition</code> instance from which 
     * <code>LoaderConfig</code> will be built.
     * 
     * @see org.jharks.workflow.Loader#load(ResponseSet, 
     *      SessionHistory, Queueable)
     */
    public void load(ResponseSet rs, SessionHistory sh, Queueable tr) {
        String itemType = setType(tr);
        
        LoaderConfig lc = new LoaderConfig(itemType, tr.getQueueQuestionId(), 
                null, tr.getFilters());
        
        load(rs, sh, lc);
     }
    
    /**
     * adds an element to the queue for each response in the list.  The stacked 
     * object represents the answer response. 
     * 
     * @param responses list of responses to be processed.
     * @param sh a <code>SessionHistory</code> instance.
     * @param config a <code>LoaderConfig</code> instance.
     */
    public void load(List<Answer> responses, SessionHistory sh, LoaderConfig config) {
        
        QueueWrapper sElement = null;
        boolean stack = false;
        String itemType = config.getItemType();
        
        if (sh.isStackEmpty() ||
            (!(sh.peek().getElementType().equalsIgnoreCase(itemType)))) {
            sElement = new QueueWrapper(itemType);
            stack = true;
        } else {
            //Category name same as top queue on stack so add to queue.
            sElement = (QueueWrapper) sh.peek();
        }
         
        for (Iterator<Answer> itr=responses.iterator(); itr.hasNext(); ) {
          Answer an = itr.next();
          String responseValue = an.getValue().toString();
          if (filter(responseValue, config)) continue;
          QueueNode qn = new QueueNode(itemType, responseValue);
          if (logger.isDebugEnabled())
            logger.debug("QueueByResponse: ItemType = [" + itemType + "]");
          sElement.add(qn);
        }
        
        //Add queue to stack 
        if ( stack && sElement.queueSize() > 0) sh.stackElement(sElement);
        
    }
}
