package org.jharks.workflow.rules;

import java.util.ArrayList;
import java.util.EmptyStackException;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jharks.workflow.QueueNode;
import org.jharks.workflow.QueueWrapper;
import org.jharks.workflow.Rule;
import org.jharks.workflow.RuleCondition;
import org.jharks.workflow.SessionHistory;
import org.jharks.workflow.Stackable;
import org.jharks.workflow.WorkFlowConstants;


/**
 * The rule will examine the queue on the <CODE>SessionHistory</CODE>
 * and return true if an item is in queue and meets any configured 
 * conditions.  If no conditions are set the rule will apply as long
 * as the queue is not empty. The rule, when applies, will dequeue a 
 * node and sets a session property.
 *  
 * @author Michael J. Rizzo
 */
public class QueueIterateRule extends Rule implements WorkFlowConstants {

    /** The logger for the module. */
    protected Log logger = LogFactory.getLog(getClass().getName());
    
    /** Conditions available on rule instance. */
    private ArrayList<RuleCondition> conditions;
    
    /** Default public constructor that initializes conditions list. */
    public QueueIterateRule() {
        conditions = new ArrayList<RuleCondition>();
    }
    
    /**
     * Checks rule properties against supplied input to see if 
     * rule must be applied.
     * @see org.jharks.workflow.Rule#checkRule(java.lang.Object)
     */
    public boolean checkRule(Object input)  {
        
       QueueNode qn = iterate(input);
       if (qn != null) {
           setSessionProperty(qn, (SessionHistory) input); 
           return true;  
       }
          
       return false;
    }
    
    /**
     * Returns the next queue node or null if there are no remaining nodes 
     * to dequeue.
     * @param input a <code>SessionHistory</code> instance.
     * @return the next dequeued node or null if no nodes remain 
     */
    protected QueueNode iterate(Object input) {
        
        if (input instanceof SessionHistory) {
            SessionHistory sh = (SessionHistory) input;
      
            if (logger.isDebugEnabled())
                logger.debug(this.getClass().getName() + " processing...");
        
        if (sh.isStackEmpty())
            return null;
        
        QueueWrapper item = (QueueWrapper) sh.peek();
        
        if (logger.isDebugEnabled())
            logger.debug(this.getClass().getName() + " name = " 
                    + item.getElementType() +
                    " queue size " + item.queueSize());       
        
        if ( item.queueSize() == 0 && 
                sh.getProperty(item.getElementType()) != null) {
            sh.popStack();
            try {
              item = (QueueWrapper) sh.peek();
            } catch (EmptyStackException ese) {
              logger.info("No nodes available on the queue: " + 
                      ese.getMessage());
              return null;
            }
        }
        
        if (conditions.size() == 0) {
            if (logger.isDebugEnabled())
                logger.debug(this.getClass().getName() + 
                        ": no conditions exist. ");
        } else {
        
          if (logger.isDebugEnabled())
              logger.debug(this.getClass().getName() 
                      + ": conditions exist: " +
                      "queue size = " + item.queueSize());
          for (int i=0; i < conditions.size(); i++) {
            RuleCondition rc = (RuleCondition) conditions.get(i);
            if (! rc.checkCondition(input)) {
              return null;
            }
          }
          
          if ( item.queueSize() == 0 )
              return null;
        }
         
        QueueNode qn = (QueueNode) item.remove();
        //TODO: revisit handling of popping stack item
        // after removal of last queue node. Currently
        // leaving on and popping at beginning of iterate
        // or clean up in WorkflowManager will pop. If pop
        // here impacts what's in context.
          
        if (logger.isDebugEnabled())
            logger.debug(this.getClass().getName() + ": iterating queue " +
                    "node: name = [" + qn.getName() + "]: queue size = " + 
                    item.queueSize());
        
          return qn;
          
        } else {
            throw new ClassCastException("Expecting SessionHistory Object.");
        }
    }

    /**
     * Add <CODE>RuleCondition</CODE> instance to <CODE>Rule</CODE>. 
     * @see org.jharks.workflow.Rule#addRuleCondition(
     *      org.jharks.workflow.RuleCondition)
     */
    public void addRuleCondition(RuleCondition rc) {
        conditions.add(rc);
    }
    
    /**
     * Retrieve conditions of the <CODE>Rule</CODE> instance.
     * @return the conditions.
     */
    public List<RuleCondition> getRuleConditions() {
        return conditions;
    }
    
    /**
     * Sets a session property.
     * @param item the <code>Stackable</code> instane from which to retrieve the 
     *        property value.
     * @param sh the <code>SessionHistory</code> object.
     */
    protected void setSessionProperty(Stackable item, SessionHistory sh) {
      if (!item.getElementType().equals(GEN_CATEGORY)) {
          if( item.getPropertyValue() != null ) {
            sh.setProperty(item.getElementType(), 
                    item.getPropertyValue());
          } else {
            sh.setProperty(item.getElementType(), 
                    ((String) item.getStackObject()));
          }
        }
    }
    
    /**
     * Sets a session property.
     * @param item the <code>QueueNode</code> from which to retrieve the 
     *        property value.
     * @param sh the <code>SessionHistory</code> object.
     */
    protected void setSessionProperty(QueueNode item, SessionHistory sh) {     
       if (logger.isDebugEnabled())
        logger.debug("Setting session property: name = [" + item.getName() +
                "] : value = [" + item.getValue() + "]");
        sh.setProperty(item.getName(), item.getValue());
    }
}
