package org.jharks.workflow.util;

import java.util.Collection;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Map;
import java.util.NoSuchElementException;


/**
 * Adapter class that wraps an <code>Enumeration</code> around a
 * collection classes <code>Iterator</code> object, so that existing APIs
 * returning Enumerations can easily run on top of the new collections.
 * Constructors are provided to easliy create such wrappers.
 *
 * @author Michael J. Rizzo
 */

public final class Enumerator implements Enumeration<String> {

    /**
     * The <code>Iterator</code> over which the <code>Enumeration</code>
     * represented by this class actually operates.
     */
    private Iterator<String> iterator = null;
    
    /**
     * Return an Enumeration over the values of the specified Collection.
     *
     * @param collection Collection whose values should be enumerated.
     */
    public Enumerator(Collection<String> collection) {
        this(collection.iterator());
    }

    /**
     * Return an Enumeration over the values returned by the
     * specified Iterator.
     *
     * @param iterator Iterator to be wrapped.
     */
    public Enumerator(Iterator<String> iterator) {
        super();
        this.iterator = iterator;
    }

    /**
     * Return an Enumeration over the values of the specified Map.
     *
     * @param map Map whose values should be enumerated.
     */
    public Enumerator(Map<Object, String> map) {
        this(map.values().iterator());
    }

    /**
     * Tests if this enumeration contains more elements.
     *
     * @return <code>true</code> if and only if this enumeration object
     *         contains at least one more element to provide, 
     *         <code>false</code> otherwise.
     */
    public boolean hasMoreElements() {
        return (iterator.hasNext());
    }

    /**
     * Returns the next element of this enumeration if this enumeration
     * has at least one more element to provide.
     *
     * @return the next element of this enumeration
     * @exception NoSuchElementException if no more elements exist
     */
    public String nextElement() throws NoSuchElementException {
        return (iterator.next());
    }

}

