package org.jharks.workflow.util;

import java.util.Locale;

import org.jharks.workflow.Answer;
import org.jharks.workflow.WorkFlowConfig;


/**
 * Class contains utility methods that are utilized by the
 * workflow manager. 
 * 
 * @author Michael J Rizzo.
 */
public class WorkFlowUtilities {

    
    /** The application configuration. */
    private static WorkFlowConfig config = WorkFlowConfig.getInstance();
    
    /** The home directory for files. */
    private static String homedir;
    
    static {
        try {
            homedir = config.getProperty("base-directory");
        } catch (Exception e) {
            homedir = null;
        }
        if (homedir == null) {
            homedir = System.getProperty("user.dir");
            String fileSeparator = System.getProperty("file.separator");
            if (! homedir.endsWith(fileSeparator)) {
                homedir = homedir + fileSeparator;
            }
        }
        
    }
    
    /**
     * Given a relative path, returns a full path to the file.
     * 
     * @param relativePath the relative path.
     * 
     * @return the full path.
     */
    public static String getFullPath(String relativePath) {
        return homedir + relativePath;
    }
    
    /**
     * Given a path to a file, returns a localized path by
     * appending the locale to the filename.  For example,
     * the path <code>/directory/file.ext</code> would be
     * converted to <code>/directory/file_en_US.ext</code> 
     * for the English, US locale.
     * 
     * @param path the path to convert.
     * @param locale the locale to use to localize the path.
     * 
     * @return a localized file path.
     */
    public static String getLocalizedPath(String path, Locale locale) {
        if (locale == null) {
            locale = Locale.getDefault();
        }
        StringBuffer newPath = new StringBuffer();
        int lastDot = path.lastIndexOf(".");
        if (lastDot >= 0) {
            newPath.append(path.substring(0, lastDot));
        } else {
            newPath.append(path);
        }
        
        newPath.append("_" + locale.getLanguage());
        newPath.append("_" + locale.getCountry());
        
        if (lastDot >= 0) {
            newPath.append(path.substring(lastDot));
        }
        return newPath.toString();
    }
    
    /**
     * Retrieve value object from response.
     * @param response the response object.
     * @return the value object.
     */
    public static Object getAnswerValue(Object response) {
        if (!(response instanceof Answer) )
            throw new ClassCastException("Response is not a Answer Object.");
        
        return ((Answer) response).getValue();
    }
    
    /**
     * Returns the class name without any packagename.
     * @param c the <CODE>Class</CODE> to inspect.
     * @return the class name without any package specification.
     */
    public static String getClassName(Class<?> c) {
        String QualifiedClassName = c.getName();
        int firstChar;
        firstChar = QualifiedClassName.lastIndexOf ('.') + 1;
        if ( firstChar > 0 ) {
          QualifiedClassName = QualifiedClassName.substring ( firstChar );
          }
        return QualifiedClassName;
    }


    /**
     * Returns the fully qualified class name.
     * @param c the <CODE>Class</CODE> to inspect.
     * @return the fully qualified class name with package specification.
     */
    public static String getFullClassName(Class<?> c) {
        return  c.getName();
    }

    
    /**
     * Return the package name for the specified class.
     * @param c the <CODE>Class</CODE> to inspect.
     * @return the package name for the specified class or an empty string if
     * class is in default package.
     */
    public static String getPackageName(Class<?> c) {
        String fullyQualifiedName = c.getName();
        int lastDot = fullyQualifiedName.lastIndexOf ('.');
        if (lastDot==-1){ return ""; }
        return fullyQualifiedName.substring (0, lastDot);
    }
}
